/*
*******************************************************
* Subroutine GMATRIX                                  *
* - Compute G matrix for each element                 *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void GMATRIX(double ECOOD[],int ELNOD[],int kMATS,
			 int kTSEL,double CLENG,double PROPS[],
			 double EGMTX[])
{
	void DUCLEAN();
	void GAUSSQU();
	void QUANGAS();
	void TREFFTZ();
	void SCHTREF();
	void MATMULT();
	void MATTRAN();
	extern int NTREF,NDIME,NDOFN,NNODE,NEDGE,NGAUS,
		NPROP,NSTRE;
	double *POSGP,*WEIGP,THICK,EXISP,*CORGS,DVOLU,
		*AMTRX,*SHMTX,*N_SET,*T_SET,*Q_SET,*TQ_SET,*QTS;
	int ii,jj,im,jn,n1,NEVAB;
	
	NEVAB=NNODE*NDOFN;
	// Gaussian point and weight coefficients
	POSGP=(double *)calloc(NGAUS,sizeof(double));
	DUCLEAN(NGAUS,1,POSGP);
	WEIGP=(double *)calloc(NGAUS,sizeof(double));
	DUCLEAN(NGAUS,1,WEIGP);
	GAUSSQU(POSGP,WEIGP);
	// Material properties
	THICK=PROPS[kMATS*NPROP+2];
	// Compute H matrix
	for(ii=0;ii<NEDGE;ii++)
	{
		for(jj=0;jj<NGAUS;jj++)
		{
			EXISP=POSGP[jj];
			// Related quantities at Gaussian point
			CORGS=(double *)calloc(1*NDIME,
				sizeof(double));
			DUCLEAN(1,NDIME,CORGS);
			AMTRX=(double *)calloc(NDOFN*NSTRE,
				sizeof(double));
			DUCLEAN(NDOFN,NSTRE,AMTRX);
			SHMTX=(double *)calloc(NDOFN*NEVAB,
				sizeof(double));
			DUCLEAN(NDOFN,NEVAB,SHMTX);
			QUANGAS(ii,EXISP,ECOOD,ELNOD,CORGS,&DVOLU,
				AMTRX,SHMTX);
			DVOLU=DVOLU*WEIGP[jj];
			if((THICK+1)!=1)
			{
				DVOLU=DVOLU*THICK;
			}
			// Trefftz functions
			N_SET=(double *)calloc(NDOFN*NTREF,
				sizeof(double));
			DUCLEAN(NDOFN,NTREF,N_SET);
			T_SET=(double *)calloc(NSTRE*NTREF,
				sizeof(double));
			DUCLEAN(NSTRE,NTREF,T_SET);
			if(kTSEL==0) // general element
			{
				TREFFTZ(CORGS[0],CORGS[1],N_SET,T_SET);
			}
			else if(kTSEL==1)
			{ // special circular hole element
				SCHTREF(CORGS[0],CORGS[1],CLENG,
					N_SET,T_SET);
            }
			// Q=A*T
			Q_SET=(double *)calloc(NDOFN*NTREF,
				sizeof(double));
			DUCLEAN(NDOFN,NTREF,Q_SET);
			MATMULT(AMTRX,T_SET,NDOFN,NSTRE,NTREF,Q_SET);
			// Q'
			TQ_SET=(double *)calloc(NTREF*NDOFN,
				sizeof(double));
			DUCLEAN(NTREF,NDOFN,TQ_SET);
			MATTRAN(Q_SET,NDOFN,NTREF,TQ_SET);
			// Q'*SHAPE
			QTS=(double *)calloc(NTREF*NEVAB,
				sizeof(double));
			DUCLEAN(NTREF,NEVAB,QTS);
			MATMULT(TQ_SET,SHMTX,NTREF,NDOFN,NEVAB,QTS);
			//
			for(im=0;im<NTREF;im++)
			{
				for(jn=0;jn<NEVAB;jn++)
				{
					n1=im*NEVAB+jn;
					EGMTX[n1]=EGMTX[n1]+QTS[n1]*DVOLU;
				}
			}
			free(CORGS); free(AMTRX);
			free(SHMTX); free(N_SET); 
			free(T_SET); free(Q_SET);
			free(TQ_SET); free(QTS);
		}
	}
	free(POSGP); free(WEIGP);
	return;
}