/*
*******************************************************
* Subroutine PVECTOR                                  *
* - Compute effective nodal force                     *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void PVECTOR(int MATNO[],double PROPS[],int LNODS[],
			 double COORD[],int NDLEG,int NEASS[],
			 int NOPRS[],double PRESS[],int NPLOD,
			 int LODPT[],double POINT[],double GP[])
{
	void GAUSSQU();
	void SHAPFUN();
	void DUCLEAN();
	extern int NDIME,NDOFN,NGAUS,NPROP,NNODE,NEDGE,
		NODEG;
	double *POSGP,*WEIGP,*ELCOD,*EPRES,*PE,*SHAPE,
		*DSHAP,*CORGS,*DERGS,*PGASH;
	int iPLOD,iDLEG,iNODE,kNODE,iODEG,iDOFN,iDIME,
		jGAUS,kPOIN,kELEM,kMATS,kEQNS,iEVAB,n1,n2,
		NEVAB,TNFEG;
	double EXISP,THICK,DVOLU;
	// Add point loads at nodes to global load vector
	if(NPLOD>0)
	{
		for(iPLOD=0;iPLOD<NPLOD;iPLOD++)
		{
			for(iDOFN=0;iDOFN<NDOFN;iDOFN++)
			{
				n1=LODPT[iPLOD]*NDOFN+iDOFN;
				n2=iPLOD*NDOFN+iDOFN;
				GP[n1]=GP[n1]+POINT[n2];
			}
		}
	}
	// Evaluate equivalent nodal force caused 
	// by distributed edge load
	NEVAB=NNODE*NDOFN;
	TNFEG=NODEG*NDOFN;
	// Gaussian point and weight coefficients
	POSGP=(double *)calloc(NGAUS,sizeof(double));
	DUCLEAN(NGAUS,1,POSGP);
	WEIGP=(double *)calloc(NGAUS,sizeof(double));
	DUCLEAN(NGAUS,1,WEIGP);
	GAUSSQU(POSGP,WEIGP);
	for(iDLEG=0;iDLEG<NDLEG;iDLEG++)
	{
		kELEM=NEASS[iDLEG];
		// Material properties
		kMATS=MATNO[kELEM];
		THICK=PROPS[kMATS*NPROP+2];
		// Determine coordinates of nodes on the edge
		ELCOD=(double *)calloc(NODEG*NDIME,
			sizeof(double));
		DUCLEAN(NODEG,NDIME,ELCOD);
		for(iODEG=0;iODEG<NODEG;iODEG++)
		{
			kPOIN=NOPRS[iDLEG*NODEG+iODEG];
			for(iDIME=0;iDIME<NDIME;iDIME++)
			{
				n1=iODEG*NDIME+iDIME;
				n2=kPOIN*NDIME+iDIME;
				ELCOD[n1]=COORD[n2];
			}
		}
		// Determine local nodal load intensity
		EPRES=(double *)calloc(NODEG*NDOFN,
			sizeof(double));
		DUCLEAN(NODEG,NDOFN,EPRES);
		for(iODEG=0;iODEG<NODEG;iODEG++)
		{
			for(iDOFN=0;iDOFN<NDOFN;iDOFN++)
			{
				n1=iODEG*NDOFN+iDOFN;
				n2=iDLEG*TNFEG+n1;
				EPRES[n1]=PRESS[n2];
			}
		}
		// Integration along the loaded edge
		PE=(double *)calloc(NEVAB,sizeof(double));
		DUCLEAN(NEVAB,1,PE);
		for(jGAUS=0;jGAUS<NGAUS;jGAUS++)
		{
			EXISP=POSGP[jGAUS];
			// shape functions and its derivatives
			SHAPE=(double *)calloc(NODEG,sizeof(double));
			DUCLEAN(1,NODEG,SHAPE);
			DSHAP=(double *)calloc(NODEG,sizeof(double));
			DUCLEAN(1,NODEG,DSHAP);
			SHAPFUN(EXISP,SHAPE,DSHAP);
			// geometric quantities of Gaussian points
			// x=sum(Ni*xi) and y=sum(Ni*yi)
			// dx/dt=sum(dNi/dt*xi)
			// dy/dt=sum(dNi/dt*yi)
			// DVOLU=sqrt((dx/dt)^2+(dy/dt)^2)
			CORGS=(double *)calloc(NDIME,
				sizeof(double));
			DUCLEAN(1,NDIME,CORGS);
			DERGS=(double *)calloc(NDIME,
				sizeof(double));
			DUCLEAN(1,NDIME,DERGS);
			for(iDIME=0;iDIME<NDIME;iDIME++)
			{
				for(iODEG=0;iODEG<NODEG;iODEG++)
				{
					n1=iODEG*NDIME+iDIME;
					CORGS[iDIME]=CORGS[iDIME]+
						SHAPE[iODEG]*ELCOD[n1];
					DERGS[iDIME]=DERGS[iDIME]+
						DSHAP[iODEG]*ELCOD[n1];
				}
			}
			DVOLU=sqrt(pow(DERGS[0],2.0)+
				pow(DERGS[1],2.0));
			// Gauss integration factor
			DVOLU=DVOLU*WEIGP[jGAUS];
			if((THICK+1)!=1)
			{
				DVOLU=DVOLU*THICK;
			}
			// flux intensity at Gauss point
			//          p=sum(Ni*pi)
			PGASH=(double *)calloc(NDOFN,
				sizeof(double));
			DUCLEAN(NDOFN,1,PGASH);
			for(iDOFN=0;iDOFN<NDOFN;iDOFN++)
			{
				for(iODEG=0;iODEG<NODEG;iODEG++)
				{
					n1=iODEG*NDOFN+iDOFN;
					PGASH[iDOFN]=PGASH[iDOFN]+
						SHAPE[iODEG]*EPRES[n1];
				}
			}
			// Compute equivalent nodal force PE
			for(iNODE=0;iNODE<NNODE;iNODE++)
			{
				kPOIN=LNODS[kELEM*NNODE+iNODE];
				if(kPOIN==NOPRS[iDLEG*NODEG+0])
				{
					// iNODE is start node of the 
					// loaded edge
					for(iODEG=0;iODEG<NODEG;iODEG++)
					{
						kNODE=iNODE+iODEG;
						if(kNODE>=NNODE)
						{
							kNODE=0;
						}
						for(iDOFN=0;iDOFN<NDOFN;iDOFN++)
						{
							n1=kNODE*NDOFN+iDOFN;
							PE[n1]=PE[n1]+SHAPE[iODEG]*
								PGASH[iDOFN]*DVOLU;
						}
					}
				}
			}
		}
		// Assemble PE into global load vector
		for(iNODE=0;iNODE<NNODE;iNODE++)
		{
			kPOIN=LNODS[kELEM*NNODE+iNODE];
			for(iDOFN=0;iDOFN<NDOFN;iDOFN++)
			{
				kEQNS=NDOFN*kPOIN+iDOFN; // global DOF
				iEVAB=NDOFN*iNODE+iDOFN; // local DOF
				GP[kEQNS]=GP[kEQNS]+PE[iEVAB];
			}
		}
	}
	free(POSGP); free(WEIGP); free(ELCOD); 
	free(EPRES); free(PE);	free(SHAPE);
	free(DSHAP); free(DERGS); free(PGASH);
	return;
}