/*
*******************************************************
* Subroutine QUANGAS                                  *
* - Evaluate quantities at Gaussian points            *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void QUANGAS(int iEDGE,double EXISP,double ECOOD[],
			 int ELNOD[],double CORGS[],double *DVOLU,
			 double AMTRX[],double SHMTX[])
{
	void SHAPFUN();
	void DUCLEAN();
	extern int NDIME,NDOFN,NNODE,NEDGE,NODEG;
	double *SHAPE,*DSHAP,*DERGS;
	int ii,jj,in,kn;

	// shape functions and its derivatives
	SHAPE=(double *)calloc(NODEG,sizeof(double));
	DUCLEAN(1,NODEG,SHAPE);
	DSHAP=(double *)calloc(NODEG,sizeof(double));
	DUCLEAN(1,NODEG,DSHAP);
	SHAPFUN(EXISP,SHAPE,DSHAP);
	// Coordinates and derivatives of Gauss points
	//   x=sum(Ni*xi) and y=sum(Ni*yi)
	//   dx/dt=sum(dNi/dt*xi), dy/dt=sum(dNi/dt*yi)
	//   DVOLU=sqrt((dx/dt)^2+(dy/dt)^2)
	DERGS=(double *)calloc(NDIME,sizeof(double));
	DUCLEAN(1,NDIME,DERGS);
	for(ii=0;ii<NDIME;ii++)
	{
		for(jj=0;jj<NODEG;jj++)
		{
			kn=ELNOD[iEDGE*NODEG+jj];
			CORGS[ii]=CORGS[ii]+
				SHAPE[jj]*ECOOD[kn*NDIME+ii];
			DERGS[ii]=DERGS[ii]+
				DSHAP[jj]*ECOOD[kn*NDIME+ii];
		}
	}
	*DVOLU=sqrt(pow(DERGS[0],2.0)+pow(DERGS[1],2.0));
	// Directional cosine at Gauss point
	//   nx = dy/dS,  ny = - dx/dS
	AMTRX[0]= DERGS[1]/(*DVOLU);
	AMTRX[1]=-DERGS[0]/(*DVOLU);
	// Form shape function matrix of frame function
	in=ELNOD[iEDGE*NODEG+0];
	for(ii=0;ii<NODEG;ii++)
	{
		kn=in+ii;
		if(kn>=NNODE)
		{
			kn=0;
		}
		SHMTX[kn]=SHAPE[ii];
	}
	return;
}