/* 
*******************************************************
* Subroutines ocdiv, oclog, ocexp, opowr, ocmul,      *
*             ocactan                                 *
* - complex algorithm                                 *
*******************************************************
*/
/* complex division (a+ib)/(c+id) */
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void ocdiv(double a,double b,double c,double d,
		   double *e,double *f)
{
	double p,q,s,w;
    p=a*c; 
	q=-b*d; 
	s=(a+b)*(c-d);
    w=c*c+d*d;
    if((1+w)==1)
	{
		printf("Error, the denominator is zero!\n");
		exit(0);
	}
	*e=(p-q)/w; 
	*f=(s-p-q)/w;
    return;
}

/* Complex logarithm ln((x+iy)) */
void oclog(double x,double y,double *u,double *v)
{
	double p;
    p=log(sqrt(x*x+y*y));
    *u=p; 
	*v=atan2(y,x);
    return;
}

/* Complex exponent exp((x+iy)) */
void ocexp(double x,double y,double *u,double *v)
{
	double p;
    p=exp(x);
    *u=p*cos(y); 
	*v=p*sin(y);
    return;
}

/* Complex power (x+iy)^n */
void ocpow(double x,double y,int n,double *u,double *v)
{
	double r,q;
	if(fabs(y/x)<1e-12)
	{
		y=0.0;
	}
    q=atan2(y,x);
    r=sqrt(x*x+y*y);
    if (r>1e-10)
	{
		r=n*log(r);
		r=exp(r);
	}
    *u=r*cos(n*q); 
	*v=r*sin(n*q);
    return;
}

/* Complex multiply (a+ib)*(c+id) */
void ocmul(double a,double b,double c,double d,
		   double *e,double *f)
{
	double p,q,s;
    p=a*c;
	q=b*d; 
	s=(a+b)*(c+d);
    *e=p-q;
	*f=s-p-q;
    return;
}

/* Complex arctan((a+ib)) */
void ocactan(double a,double b,double *c,double *d)
{
  void ocdiv();
  void oclog();
  double e,f,u,v;
  ocdiv(1-b,a,1+b,-a,&e,&f);
  oclog(e,f,&u,&v);
  *c=v/2;
  *d=-u/2;
}