/*
*******************************************************
* Subroutine FIEDCEN                                  *
* -Compute related fields at centroid of each element *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void FIEDCEN(int NELEM,int MATNO[],int LNODS[],
			 double COORD[],double PROPS[],int ELNOD[],
			 double ASDIS[],int NINTP,double IPCOD[],
			 double CRBFI[],double CECOD[],
			 double UCENP[],double SCENP[])
{
	void ELEPARS(); void DUCLEAN();	void HMATRIX();
	void GMATRIX();	void EDISNOD();	void CMATRIX();
	void RIGIDRV();	void TREFFTZ();	void MATMULT();
	void PARSOLU(); void USERFUN();
	extern int NTREF,NNODE,NDIME,NDOFN,NSTRE,NTYPE,NPROP;
	int iELEM,kMATS,NEVAB;
	double *ECOOD,*CenCoord,*EHMTX,*EGMTX,*d_Ele,
		*c_Ele,*c0,*N_SET,*T_SET,*GDISP,*GSTRE,
		xp,yp,YOUNG,POISS,THICK,TEXPN,DENST,
		gm,gxp,gyp,*ups,*sps,*GBF,TEM;
	
    // Material properties
	YOUNG=PROPS[0*NPROP+0];
	POISS=PROPS[0*NPROP+1];
	THICK=PROPS[0*NPROP+2];
	TEXPN=PROPS[0*NPROP+3];
	DENST=PROPS[0*NPROP+4];
	if(NTYPE==1) //plane stress
	{
		gm=YOUNG*TEXPN/(1.0-POISS);
	}
	else if(NTYPE==2)
	{
		gm=YOUNG*TEXPN/(1.0-2*POISS);
	}

	NEVAB=NNODE*NDOFN;
	for(iELEM=0;iELEM<NELEM;iELEM++)
	{
		kMATS=MATNO[iELEM];
		// Compute some quantities related to each element
		ECOOD=(double *)calloc(NNODE*NDIME,sizeof(double));
		DUCLEAN(NNODE,NDIME,ECOOD);
		CenCoord=(double *)calloc(1*NDIME,sizeof(double));
		DUCLEAN(1,NDIME,CenCoord);
		ELEPARS(iELEM,LNODS,COORD,ECOOD,CenCoord);
		// Compute H matrix
		EHMTX=(double *)calloc(NTREF*NTREF,sizeof(double));
		DUCLEAN(NTREF,NTREF,EHMTX);
		HMATRIX(ECOOD,ELNOD,kMATS,PROPS,EHMTX);
		// Compute G matrix
		EGMTX=(double *)calloc(NTREF*NEVAB,sizeof(double));
		DUCLEAN(NTREF,NEVAB,EGMTX);
		GMATRIX(ECOOD,ELNOD,kMATS,PROPS,EGMTX);
		// Nodal displacements
		d_Ele=(double *)calloc(NEVAB,sizeof(double));
		DUCLEAN(NEVAB,1,d_Ele);
		EDISNOD(iELEM,LNODS,ASDIS,d_Ele);
		// Calculate the ce coefficients
		c_Ele=(double *)calloc(NTREF*1,sizeof(double));
		DUCLEAN(NTREF,1,c_Ele);
		CMATRIX(EHMTX,EGMTX,d_Ele,c_Ele);
		// Recover rigid displacement
		c0=(double *)calloc(3*1,sizeof(double));
		DUCLEAN(3,1,c0);
		RIGIDRV(ECOOD,c_Ele,d_Ele,kMATS,PROPS,c0);
		// Compute Trefftz internal fields at central point
		N_SET=(double *)calloc(NDOFN*NTREF,sizeof(double));
		DUCLEAN(NDOFN,NTREF,N_SET);
		T_SET=(double *)calloc(NSTRE*NTREF,sizeof(double));
		DUCLEAN(NSTRE,NTREF,T_SET);
		xp=0;
		yp=0;
		TREFFTZ(xp,yp,kMATS,PROPS,N_SET,T_SET);
		GDISP=(double *)calloc(NDOFN*1,sizeof(double));
		DUCLEAN(NDOFN,1,GDISP);
		GSTRE=(double *)calloc(NSTRE*1,sizeof(double));
		DUCLEAN(NSTRE,1,GSTRE);
		MATMULT(N_SET,c_Ele,NDOFN,NTREF,1,GDISP);
		MATMULT(T_SET,c_Ele,NSTRE,NTREF,1,GSTRE);
		// particular solution
		gxp=CenCoord[0]+xp;
		gyp=CenCoord[1]+yp;
		ups=(double *)calloc(NDOFN,sizeof(double));
		DUCLEAN(NDOFN,1,ups);
		sps=(double *)calloc(NSTRE,sizeof(double));
		DUCLEAN(NSTRE,1,sps);
		PARSOLU(gxp,gyp,NINTP,IPCOD,CRBFI,PROPS,ups,sps);
		// Temperature effect
		GBF=(double *)calloc(NDOFN,sizeof(double));
		DUCLEAN(NDOFN,1,GBF);
		USERFUN(gxp,gyp,PROPS,GBF,&TEM);
		// Full solution
		UCENP[iELEM*NDOFN+0]=GDISP[0]+c0[0]+yp*c0[2]+ups[0];
		UCENP[iELEM*NDOFN+1]=GDISP[1]+c0[1]-xp*c0[2]+ups[1];
		SCENP[iELEM*NSTRE+0]=GSTRE[0]+sps[0]-gm*TEM;
		SCENP[iELEM*NSTRE+1]=GSTRE[1]+sps[1]-gm*TEM;
		SCENP[iELEM*NSTRE+2]=GSTRE[2]+sps[2];
		// Coordinates of commputing point
		CECOD[iELEM*NDIME+0]=gxp;
		CECOD[iELEM*NDIME+1]=gyp;
	}
	free(ECOOD); free(CenCoord); free(EHMTX);
	free(EGMTX); free(d_Ele); free(c_Ele);
	free(N_SET); free(T_SET); free(GDISP);
	free(GSTRE);
}
