/* Subroutine LASS: Solve linear algebric system */
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void LAS_SVD(double A[],double b[],int n)
{
	void DUCLEAN();
	void SVDM();
	void EQUIL();
	double RESIDUAL();
	int i;
	double *x,conda,norm;

	x=(double *)calloc(n,sizeof(double));
	DUCLEAN(n,1,x);
	/* Equilibration treatment */
	EQUIL(A,b,n);
	/* SVD for A x = b */
	SVDM(A,b,n,&conda,x);
	/* Compute residual value of vector ||b-Ax|| */
	norm=RESIDUAL(A,b,x,n);
	printf("Condition number = %5.3e\n",conda);
	printf("Residual norm    = %5.3e\n",norm);
	for(i=0;i<n;i++)
	{
		b[i]=x[i];
	}
	free(x);
	return;
}


/* SVD for A x = b */
void SVDM(double A[],double b[],int n,double *cond,double x[])
{
	void DUCLEAN();
	int SVDCMP();
	void ZEROW();
	void SVDKSB();
	double *U,*W,*VT,wmax,wmin;
	int i,j;

	U=(double *)calloc(n*n,sizeof(double));
	W=(double *)calloc(n*n,sizeof(double));
	VT=(double *)calloc(n*n,sizeof(double));
	DUCLEAN(n,n,U);
	DUCLEAN(n,n,W);
	DUCLEAN(n,n,VT);
	
	for(i=0;i<n;i++) // Copy A into W if you don't want matrix A to be destroyed
	{
		for(j=0;j<n;j++)
		{
			W[i*n+j]=A[i*n+j];
		}
	}
	/* Singular Value Decomposition of Matrix A */
	i=SVDCMP(W,n,n,U,VT,n+1);
	if(i<0)
	{
		printf("Fail! Number of iteration exceeds specified maxinum number\n");
		exit(0);
	}
	/* Determine the largest singular value */
	wmax=0.0;
	for(i=0;i<n;i++)
	{
		if(W[i*n+i]>wmax)
		{
			wmax=W[i*n+i];
		}
	}
	/* Determine the smallest singular value */
	wmin=wmax;
	for(i=0;i<n;i++)
	{
		if(W[i*n+i]<wmin)
		{
			wmin=W[i*n+i];
		}
	}
	/* Condition number of matrix A*/
	*cond=fabs(wmax/wmin);
	//-------Zeroing smaller singular values-------
	ZEROW(W,n);
    /* Backsubstitute to solve A x = b */
	SVDKSB(U,W,VT,n,b,x); // Vector b can not be destroyed
	free(U);
	free(W);
	free(VT);
	return;
}


/* Sunroutine SVDCMP: singular value decomposition of real matrix A */
int SVDCMP(double a[],int m,int n,double u[],double v[],int ka)
{
	void ppp();
    void sss();
	int i,j,k,l,it,ll,kk,ix,iy,mm,nn,iz,m1,ks;
    double d,dd,t,sm,sm1,em1,sk,ek,b,c,shh,fg[2],cs[2];
    double *s,*e,*w;
	double eps=1.0e-15;
    s=malloc(ka*sizeof(double));
    e=malloc(ka*sizeof(double));
    w=malloc(ka*sizeof(double));
    it=60; 
	k=n;
    if (m-1<n) k=m-1;
    l=m;
    if (n-2<m) l=n-2;
    if (l<0) l=0;
    ll=k;
    if (l>k) ll=l;
    if (ll>=1)
	{
		for (kk=1; kk<=ll; kk++)
          { if (kk<=k)
              { d=0.0;
                for (i=kk; i<=m; i++)
                  { ix=(i-1)*n+kk-1; d=d+a[ix]*a[ix];}
                s[kk-1]=sqrt(d);
                if (s[kk-1]!=0.0)
                  { ix=(kk-1)*n+kk-1;
                    if (a[ix]!=0.0)
                      { s[kk-1]=fabs(s[kk-1]);
                        if (a[ix]<0.0) s[kk-1]=-s[kk-1];
                      }
                    for (i=kk; i<=m; i++)
                      { iy=(i-1)*n+kk-1;
                        a[iy]=a[iy]/s[kk-1];
                      }
                    a[ix]=1.0+a[ix];
                  }
                s[kk-1]=-s[kk-1];
              }
            if (n>=kk+1)
              { for (j=kk+1; j<=n; j++)
                  { if ((kk<=k)&&(s[kk-1]!=0.0))
                      { d=0.0;
                        for (i=kk; i<=m; i++)
                          { ix=(i-1)*n+kk-1;
                            iy=(i-1)*n+j-1;
                            d=d+a[ix]*a[iy];
                          }
                        d=-d/a[(kk-1)*n+kk-1];
                        for (i=kk; i<=m; i++)
                          { ix=(i-1)*n+j-1;
                            iy=(i-1)*n+kk-1;
                            a[ix]=a[ix]+d*a[iy];
                          }
                      }
                    e[j-1]=a[(kk-1)*n+j-1];
                  }
              }
            if (kk<=k)
              { for (i=kk; i<=m; i++)
                  { ix=(i-1)*m+kk-1; iy=(i-1)*n+kk-1;
                    u[ix]=a[iy];
                  }
              }
            if (kk<=l)
              { d=0.0;
                for (i=kk+1; i<=n; i++)
                  d=d+e[i-1]*e[i-1];
                e[kk-1]=sqrt(d);
                if (e[kk-1]!=0.0)
                  { if (e[kk]!=0.0)
                      { e[kk-1]=fabs(e[kk-1]);
                        if (e[kk]<0.0) e[kk-1]=-e[kk-1];
                      }
                    for (i=kk+1; i<=n; i++)
                      e[i-1]=e[i-1]/e[kk-1];
                    e[kk]=1.0+e[kk];
                  }
                e[kk-1]=-e[kk-1];
                if ((kk+1<=m)&&(e[kk-1]!=0.0))
                  { for (i=kk+1; i<=m; i++) w[i-1]=0.0;
                    for (j=kk+1; j<=n; j++)
                      for (i=kk+1; i<=m; i++)
                        w[i-1]=w[i-1]+e[j-1]*a[(i-1)*n+j-1];
                    for (j=kk+1; j<=n; j++)
                      for (i=kk+1; i<=m; i++)
                        { ix=(i-1)*n+j-1;
                          a[ix]=a[ix]-w[i-1]*e[j-1]/e[kk];
                        }
                  }
                for (i=kk+1; i<=n; i++)
                  v[(i-1)*n+kk-1]=e[i-1];
              }
          }
      }
    mm=n;
    if (m+1<n) mm=m+1;
    if (k<n) s[k]=a[k*n+k];
    if (m<mm) s[mm-1]=0.0;
    if (l+1<mm) e[l]=a[l*n+mm-1];
    e[mm-1]=0.0;
    nn=m;
    if (m>n) nn=n;
    if (nn>=k+1)
      { for (j=k+1; j<=nn; j++)
          { for (i=1; i<=m; i++)
              u[(i-1)*m+j-1]=0.0;
            u[(j-1)*m+j-1]=1.0;
          }
      }
    if (k>=1)
      { for (ll=1; ll<=k; ll++)
          { kk=k-ll+1; iz=(kk-1)*m+kk-1;
            if (s[kk-1]!=0.0)
              { if (nn>=kk+1)
                  for (j=kk+1; j<=nn; j++)
                    { d=0.0;
                      for (i=kk; i<=m; i++)
                        { ix=(i-1)*m+kk-1;
                          iy=(i-1)*m+j-1;
                          d=d+u[ix]*u[iy]/u[iz];
                        }
                      d=-d;
                      for (i=kk; i<=m; i++)
                        { ix=(i-1)*m+j-1;
                          iy=(i-1)*m+kk-1;
                          u[ix]=u[ix]+d*u[iy];
                        }
                    }
                  for (i=kk; i<=m; i++)
                    { ix=(i-1)*m+kk-1; u[ix]=-u[ix];}
                  u[iz]=1.0+u[iz];
                  if (kk-1>=1)
                    for (i=1; i<=kk-1; i++)
                      u[(i-1)*m+kk-1]=0.0;
              }
            else
              { for (i=1; i<=m; i++)
                  u[(i-1)*m+kk-1]=0.0;
                u[(kk-1)*m+kk-1]=1.0;
              }
          }
      }
    for (ll=1; ll<=n; ll++)
      { kk=n-ll+1; iz=kk*n+kk-1;
        if ((kk<=l)&&(e[kk-1]!=0.0))
          { for (j=kk+1; j<=n; j++)
              { d=0.0;
                for (i=kk+1; i<=n; i++)
                  { ix=(i-1)*n+kk-1; iy=(i-1)*n+j-1;
                    d=d+v[ix]*v[iy]/v[iz];
                  }
                d=-d;
                for (i=kk+1; i<=n; i++)
                  { ix=(i-1)*n+j-1; iy=(i-1)*n+kk-1;
                    v[ix]=v[ix]+d*v[iy];
                  }
              }
          }
        for (i=1; i<=n; i++)
          v[(i-1)*n+kk-1]=0.0;
        v[iz-n]=1.0;
      }
    for (i=1; i<=m; i++)
    for (j=1; j<=n; j++)
      a[(i-1)*n+j-1]=0.0;
    m1=mm; it=60;
    while (1==1)
      { if (mm==0)
          { ppp(a,e,s,v,m,n);
            free(s); free(e); free(w); return(1);
          }
        if (it==0)
          { ppp(a,e,s,v,m,n);
            free(s); free(e); free(w); return(-1);
          }
        kk=mm-1;
	while ((kk!=0)&&(fabs(e[kk-1])!=0.0))
          { d=fabs(s[kk-1])+fabs(s[kk]);
            dd=fabs(e[kk-1]);
            if (dd>eps*d) kk=kk-1;
            else e[kk-1]=0.0;
          }
        if (kk==mm-1)
          { kk=kk+1;
            if (s[kk-1]<0.0)
              { s[kk-1]=-s[kk-1];
                for (i=1; i<=n; i++)
                  { ix=(i-1)*n+kk-1; v[ix]=-v[ix];}
              }
            while ((kk!=m1)&&(s[kk-1]<s[kk]))
              { d=s[kk-1]; s[kk-1]=s[kk]; s[kk]=d;
                if (kk<n)
                  for (i=1; i<=n; i++)
                    { ix=(i-1)*n+kk-1; iy=(i-1)*n+kk;
                      d=v[ix]; v[ix]=v[iy]; v[iy]=d;
                    }
                if (kk<m)
                  for (i=1; i<=m; i++)
                    { ix=(i-1)*m+kk-1; iy=(i-1)*m+kk;
                      d=u[ix]; u[ix]=u[iy]; u[iy]=d;
                    }
                kk=kk+1;
              }
            it=60;
            mm=mm-1;
          }
        else
          { ks=mm;
            while ((ks>kk)&&(fabs(s[ks-1])!=0.0))
              { d=0.0;
                if (ks!=mm) d=d+fabs(e[ks-1]);
                if (ks!=kk+1) d=d+fabs(e[ks-2]);
                dd=fabs(s[ks-1]);
                if (dd>eps*d) ks=ks-1;
                else s[ks-1]=0.0;
              }
            if (ks==kk)
              { kk=kk+1;
                d=fabs(s[mm-1]);
                t=fabs(s[mm-2]);
                if (t>d) d=t;
                t=fabs(e[mm-2]);
                if (t>d) d=t;
                t=fabs(s[kk-1]);
                if (t>d) d=t;
                t=fabs(e[kk-1]);
                if (t>d) d=t;
                sm=s[mm-1]/d; sm1=s[mm-2]/d;
                em1=e[mm-2]/d;
                sk=s[kk-1]/d; ek=e[kk-1]/d;
                b=((sm1+sm)*(sm1-sm)+em1*em1)/2.0;
                c=sm*em1; c=c*c; shh=0.0;
                if ((b!=0.0)||(c!=0.0))
                  { shh=sqrt(b*b+c);
                    if (b<0.0) shh=-shh;
                    shh=c/(b+shh);
                  }
                fg[0]=(sk+sm)*(sk-sm)-shh;
                fg[1]=sk*ek;
                for (i=kk; i<=mm-1; i++)
                  { sss(fg,cs);
                    if (i!=kk) e[i-2]=fg[0];
                    fg[0]=cs[0]*s[i-1]+cs[1]*e[i-1];
                    e[i-1]=cs[0]*e[i-1]-cs[1]*s[i-1];
                    fg[1]=cs[1]*s[i];
                    s[i]=cs[0]*s[i];
                    if ((cs[0]!=1.0)||(cs[1]!=0.0))
                      for (j=1; j<=n; j++)
                        { ix=(j-1)*n+i-1;
                          iy=(j-1)*n+i;
                          d=cs[0]*v[ix]+cs[1]*v[iy];
                          v[iy]=-cs[1]*v[ix]+cs[0]*v[iy];
                          v[ix]=d;
                        }
                    sss(fg,cs);
                    s[i-1]=fg[0];
                    fg[0]=cs[0]*e[i-1]+cs[1]*s[i];
                    s[i]=-cs[1]*e[i-1]+cs[0]*s[i];
                    fg[1]=cs[1]*e[i];
                    e[i]=cs[0]*e[i];
                    if (i<m)
                      if ((cs[0]!=1.0)||(cs[1]!=0.0))
                        for (j=1; j<=m; j++)
                          { ix=(j-1)*m+i-1;
                            iy=(j-1)*m+i;
                            d=cs[0]*u[ix]+cs[1]*u[iy];
                            u[iy]=-cs[1]*u[ix]+cs[0]*u[iy];
                            u[ix]=d;
                          }
                  }
                e[mm-2]=fg[0];
                it=it-1;
              }
            else
              { if (ks==mm)
                  { kk=kk+1;
                    fg[1]=e[mm-2]; e[mm-2]=0.0;
                    for (ll=kk; ll<=mm-1; ll++)
                      { i=mm+kk-ll-1;
                        fg[0]=s[i-1];
                        sss(fg,cs);
                        s[i-1]=fg[0];
                        if (i!=kk)
                          { fg[1]=-cs[1]*e[i-2];
                            e[i-2]=cs[0]*e[i-2];
                          }
                        if ((cs[0]!=1.0)||(cs[1]!=0.0))
                          for (j=1; j<=n; j++)
                            { ix=(j-1)*n+i-1;
                              iy=(j-1)*n+mm-1;
                              d=cs[0]*v[ix]+cs[1]*v[iy];
                              v[iy]=-cs[1]*v[ix]+cs[0]*v[iy];
                              v[ix]=d;
                            }
                      }
                  }
                else
                  { kk=ks+1;
                    fg[1]=e[kk-2];
                    e[kk-2]=0.0;
                    for (i=kk; i<=mm; i++)
                      { fg[0]=s[i-1];
                        sss(fg,cs);
                        s[i-1]=fg[0];
                        fg[1]=-cs[1]*e[i-1];
                        e[i-1]=cs[0]*e[i-1];
                        if ((cs[0]!=1.0)||(cs[1]!=0.0))
                          for (j=1; j<=m; j++)
                            { ix=(j-1)*m+i-1;
                              iy=(j-1)*m+kk-2;
                              d=cs[0]*u[ix]+cs[1]*u[iy];
                              u[iy]=-cs[1]*u[ix]+cs[0]*u[iy];
                              u[ix]=d;
                            }
                      }
                  }
              }
          }
      }

    free(s);
	free(w);
	free(e);
	return(1);
}

void ppp(double a[],double e[],double s[],double v[],int m,int n)
{ 
	int i,j,p,q;
    double d;
    if (m>=n)
	{
		i=n;
	}
    else
	{
		i=m;
	}
    for (j=1; j<=i-1; j++)
	{
		a[(j-1)*n+j-1]=s[j-1];
        a[(j-1)*n+j]=e[j-1];
	}
    a[(i-1)*n+i-1]=s[i-1];
    if (m<n) 
	{
		a[(i-1)*n+i]=e[i-1];
	}
    for (i=1; i<=n-1; i++)
	{
		for (j=i+1; j<=n; j++)
		{
			p=(i-1)*n+j-1;
			q=(j-1)*n+i-1;
            d=v[p]; 
			v[p]=v[q];
			v[q]=d;
		}
	}
    return;
}

void sss(double fg[2],double cs[2])
{
	double r,d;
    if ((fabs(fg[0])+fabs(fg[1]))==0.0)
	{
		cs[0]=1.0; cs[1]=0.0; d=0.0;
	}
    else 
	{
		d=sqrt(fg[0]*fg[0]+fg[1]*fg[1]);
        if (fabs(fg[0])>fabs(fg[1]))
          { d=fabs(d);
            if (fg[0]<0.0) d=-d;
          }
        if (fabs(fg[1])>=fabs(fg[0]))
          { d=fabs(d);
            if (fg[1]<0.0) d=-d;
          }
        cs[0]=fg[0]/d; cs[1]=fg[1]/d;
	}
    r=1.0;
    if (fabs(fg[0])>fabs(fg[1])) r=cs[1];
    else
      if (cs[0]!=0.0) r=1.0/cs[0];
    fg[0]=d; 
	fg[1]=r;
    return;
}


/* Subroutine ZEROW: Zeroing smaller singular values */
void ZEROW(double W[],int n)
{
	int i;
	double wmax,threshold;
	double TOL=1.0e-12;
	/* Determine the largest singular value */
	wmax=0.0;
	for(i=0;i<n;i++)
	{
		if(W[i*n+i]>wmax)
		{
			wmax=W[i*n+i];
		}
	}
	/* Give threshold value */
	threshold=wmax*TOL;
	/* Zero smaller singular values */
	for(i=0;i<n;i++)
	{
		if(W[i*n+i]<threshold)
		{
			W[i*n+i]=0.0;
		}
	}
	return;
}


/* Subroutine SVDKSB: Backsubstitute to solve Ax=b */
void SVDKSB(double U[],double W[],double VT[],int n,double b[],double x[])
{
	void DUCLEAN();
	int j,i;
	double s,*temp;

	temp=(double *)calloc(n,sizeof(double));
	DUCLEAN(n,1,temp);

	for(j=0;j<n;j++)   // Calculate UT.b
	{
		s=0.0;
		if(W[j*n+j])       // Nonzero result only if W[j][j] is nonzero
		{
			for(i=0;i<n;i++)
			{
				s=s+U[i*n+j]*b[i];  // UT[j][i]=U[i][j]
			}
			s=s/W[j*n+j];  // This is the divide by W[j][j]
		}
		temp[j]=s;
	}
	for(i=0;i<n;i++)  // Matrix multiply by V to get solution vector x
	{
		s=0.0;
		for(j=0;j<n;j++) 
		{
			s=s+VT[j*n+i]*temp[j]; // V[i][j]=VT[j][i]
		}
		x[i]=s;
	}
	free(temp);
	return;
}

/* Compute residual norm : ||r||=||b-Ax|| */
double RESIDUAL(double A[],double b[],double x[],int n)
{
   void DUCLEAN();
   int i,j;
   double norm,*r;
   if(n<1)
   {
	   printf(" Fail! --Invalid matrix due to n<1--\n");
	   exit(0);
   }
   r=(double *)calloc(n,sizeof(double));
   DUCLEAN(n,1,r);
   for(i=0;i<n;i++)
   {
	   for(j=0;j<n;j++)
	   {
		   r[i]=r[i]+A[i*n+j]*x[j];
	   }
	   r[i]=b[i]-r[i];
   }
   norm=0.0;
   for(i=0;i<n;i++)
   {
	   if(fabs(r[i])>norm)
	   {
		   norm=fabs(r[i]);
	   }
   }
   free(r);
   return norm;
}