/*
*******************************************************
* Subroutine MATTRAN, MATMULT, MATINVE                *
* - Matrix transpose, multiply and inverse operations *
*******************************************************
*/
/* matrix transpose */
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void MATTRAN(double a[],int m,int n,double b[])
{// a[m][n]   b[n][m]
	int i,j;
	for(i=0;i<n;i++)
	{
		for(j=0;j<m;j++)
		{
			b[i*m+j]=a[j*n+i];
		}
	}
	return;
}


/* matrix multiply */
void MATMULT(double a[],double b[],int m,int n,
			 int k,double c[])
{
	// a[m][n] * b[n][k] = c[m][k]
	int i,j,l;
    for(i=0;i<=m-1;i++)
	{
		for(j=0;j<=k-1;j++)
		{
			c[i*k+j]=0.0;
			for(l=0;l<=n-1;l++)
			{
				c[i*k+j]=c[i*k+j]+a[i*n+l]*b[l*k+j];
			}
		}
	}
    return;
}


/* inverse matrix */
void MATINVE(double a[],int n)
{
	int *is,*js,i,j,k,l,u,v;
    double d,p,eps;
	eps=1.0e-20;
    is=malloc(n*sizeof(int));
    js=malloc(n*sizeof(int));
    for(k=0;k<=n-1;k++)
    {
		d=0.0;
        for(i=k;i<=n-1;i++)
		{
			for(j=k;j<=n-1;j++)
			{ 
				l=i*n+j; 
				p=fabs(a[l]);
				if(p>d) 
				{ 
					d=p; 
					is[k]=i;
					js[k]=j;
				}
			}
		}
        if((d+1)==1)
        {
			free(is); 
			free(js); 
			printf("**The inversion of matrix fail!\n");
            exit(0);
        }
        if(is[k]!=k)
		{
			for(j=0;j<=n-1;j++)
            {
				u=k*n+j;
				v=is[k]*n+j;
                p=a[u];
				a[u]=a[v];
				a[v]=p;
            }
		}
        if(js[k]!=k)
		{
			for(i=0;i<=n-1;i++)
            {
				u=i*n+k;
				v=i*n+js[k];
                p=a[u]; 
				a[u]=a[v];
				a[v]=p;
            }
		}
        l=k*n+k;
        a[l]=1.0/a[l];
        for(j=0;j<=n-1;j++)
		{
			if(j!=k)
            { 
				u=k*n+j;
				a[u]=a[u]*a[l];
			}
		}
        for(i=0;i<=n-1;i++)
		{ 
			if(i!=k)
			{
				for(j=0;j<=n-1;j++)
				{
					if (j!=k)
					{ 
						u=i*n+j;
					    a[u]=a[u]-a[i*n+k]*a[k*n+j];
					}
				}
			}
		}
        for(i=0;i<=n-1;i++)
		{
			if(i!=k)
            { 
				u=i*n+k; 
				a[u]=-a[u]*a[l];
			}
		}
    }
    for(k=n-1;k>=0;k--)
    {
		if (js[k]!=k)
		{
			for(j=0;j<=n-1;j++)
            { 
				u=k*n+j; 
				v=js[k]*n+j;
                p=a[u];
				a[u]=a[v];
				a[v]=p;
            }
		}
        if(is[k]!=k)
        {
			for(i=0;i<=n-1;i++)
            {
				u=i*n+k; 
				v=i*n+is[k];
                p=a[u];
				a[u]=a[v]; 
				a[v]=p;
            }
		}
    }
    free(is);
	free(js);
    return;
}