/*
*******************************************************
* Subroutine RBFINTP                                  *
* -Compute coefficients of RBF interpolation          *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void RBFINTP(int NPOIN,int NELEM,double COORD[],
			 int LNODS[],double PROPS[],int NINTP,
			 double IPCOD[],double bvect[])
{
	void DUCLEAN();
	void LAS_SVD();
	void USERFUN();
	extern int NDIME,NDOFN,NNODE,NPROP,NTYPE;
	int iP,iD,iE,iN,kP,iT,jT,n1,n,NTVAR,n2;
	double *sum0,*FMATX,xi,yi,xj,yj,r,lnr,temp,
		YOUNG,POISS,THICK,TEXPN,DENST,gm,*GBF,TEM;

	// Material properties
	YOUNG=PROPS[0*NPROP+0];
	POISS=PROPS[0*NPROP+1];
	THICK=PROPS[0*NPROP+2];
	TEXPN=PROPS[0*NPROP+3];
	DENST=PROPS[0*NPROP+4];
	if(NTYPE==1) //plane stress
	{
		gm=YOUNG*TEXPN/(1.0-POISS);
	}
	else if(NTYPE==2)
	{
		gm=YOUNG*TEXPN/(1.0-2*POISS);
	}

    // Generate interpolation points: node+centroid
	for(iP=0;iP<NPOIN;iP++)
	{
		for(iD=0;iD<NDIME;iD++)
		{
			IPCOD[iP*NDIME+iD]=COORD[iP*NDIME+iD];
		}
	}
	sum0=(double *)calloc(NDIME,sizeof(double));
	for(iE=0;iE<NELEM;iE++)
	{
		n1=NPOIN+iE;
		// Form nodal coordinates of the given element
		DUCLEAN(1,NDIME,sum0);
		for(iD=0;iD<NDIME;iD++)
		{
			for(iN=0;iN<NNODE;iN++)
			{
				kP=LNODS[iE*NNODE+iN];
				sum0[iD]=sum0[iD]+COORD[kP*NDIME+iD];
			}
			IPCOD[n1*NDIME+iD]=sum0[iD]/NNODE;
		}
	}
	// Form coefficient matrix
	NTVAR=NINTP*NDOFN;
	FMATX=(double *)calloc(NTVAR*NTVAR,sizeof(double));
	DUCLEAN(NTVAR,NTVAR,FMATX);
	for(iT=0;iT<NINTP;iT++)
	{
		xi=IPCOD[iT*NDIME+0];
		yi=IPCOD[iT*NDIME+1];
		// compute right-hand term b
		GBF=(double *)calloc(NDOFN*1,sizeof(double));
		DUCLEAN(NDOFN,1,GBF);
		USERFUN(xi,yi,PROPS,GBF,&TEM);
		for(iD=0;iD<NDOFN;iD++)
		{
			bvect[iT*NDOFN+iD]=GBF[iD];
		}
		//bvect[iT*NDOFN+0]=GBF[0];
		//bvect[iT*NDOFN+1]=GBF[1];
		//
        for(jT=0;jT<NINTP;jT++)
		{
			xj=IPCOD[jT*NDIME+0];
			yj=IPCOD[jT*NDIME+1];
			r=sqrt(pow((xi-xj),2)+pow((yi-yj),2));
			// TPS: r^(2*n)ln(r)
			n=2;
			if((1+r)==1)
			{
				r=0.0;
				lnr=0.0;
			}
			else
			{
				lnr=log(r);
			}
			temp=pow(r,(2*n))*lnr;
			for(iD=0;iD<NDOFN;iD++)
			{
				n1=(iT*NDOFN+iD)*NTVAR+(jT*NDOFN+iD);
				FMATX[n1]=temp;
			}
			/*
			n1=(iT*NDOFN+0)*NTVAR+(jT*NDOFN+0);
			FMATX[n1]=temp;
			n2=(iT*NDOFN+1)*NTVAR+(jT*NDOFN+1);
			FMATX[n2]=temp;
			*/
		}
	}
	// Solve linear algebraic equations
	LAS_SVD(FMATX,bvect,NTVAR);
	free(FMATX); free(sum0);
	return;
}
