/*
*******************************************************
* Subroutine ELEPARS                                  *
* - Compute geometric quantities related to element   *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void SCHTREF(double sp,double tp,double b,
			 int kMATS,double PROPS[],double N_SET[],
			 double T_SET[])
{
	void MATMULT();
	void DUCLEAN();
	void ITCLEAN();
	extern int NTREF,NTYPE,NNODE,NDOFN,NPROP;
	double temp,*TEMP,YOUNG,POISS,E,nu,r,s,
		A0,DF0,DG0,SF0,SG0,SH0,A1,A2,D1,D2,D3,D4,
		DF1,DG1,SF1,SG1,SH1,B1,B2,B3,B4,B5,B6,B7,
		J1,J2,J3,J4,J5,J6,J7,J8,J9,J10,J11,J12,
		DF2,DG2,DF3,DG3,SF2,SG2,SH2,SF3,SG3,SH3,
		*DM,*SM;
	int *NK,k,nt,i,j;

    // Check the number of terms of Trefftz functions
	temp=NNODE*NDOFN-3;
	if(NTREF<temp)
	{
		printf("Too small terms of Trefftz functions!\n");
	}
	// Material properties
	YOUNG=PROPS[kMATS*NPROP+0];
	POISS=PROPS[kMATS*NPROP+1];
	if(NTYPE==1) // plane stress
	{
		E=YOUNG;
		nu=POISS;
	}
	else if(NTYPE==2) // plane strain
	{
		E=YOUNG/(1-pow(POISS,2));
		nu=POISS/(1-POISS);
	}
	// Evaluate r ans s
	r=sqrt(pow(sp,2)+pow(tp,2));
	s=atan2(tp,sp);
	// k=0
	NK=(int *)calloc(NTREF,sizeof(int));
	ITCLEAN(NTREF,1,NK);
	k=0;
	A0=(1+nu)/(1-nu)*pow(b,2);
	
	DF0=A0/r+r;
	DG0=0;
	SF0=E/(1-nu)-E/(1+nu)*A0/pow(r,2);
	SG0=E/(1-nu)+E/(1+nu)*A0/pow(r,2);
	SH0=0;
	
	N_SET[0*NTREF+0]=DF0;
	N_SET[1*NTREF+0]=DG0;
	T_SET[0*NTREF+0]=SF0;
	T_SET[1*NTREF+0]=SG0;
	T_SET[2*NTREF+0]=SH0;
	// k=1
	k=1;
	A1=(1+nu)/(1-3*nu)*pow(b,4);
	A2=(5+nu)/(1-3*nu);
	D1=-2*E*A1/(1+nu);
	D2=E*(2+nu+nu*A2)/(1-pow(nu,2));
	D3=E*(1+2*nu+A2)/(1-pow(nu,2));
	D4=E*(-1+A2)/2.0/(1+nu);
	
	DF1=A1/pow(r,2)+pow(r,2);
	DG1=A1/pow(r,2)+A2*pow(r,2);
	SF1= D1*pow(r,(-3))+D2*r;
	SG1=-D1*pow(r,(-3))+D3*r;
	SH1= D1*pow(r,(-3))+D4*r;
	
	N_SET[0*NTREF+1]=DF1*cos(s);
	N_SET[1*NTREF+1]=DG1*sin(s);
	T_SET[0*NTREF+1]=SF1*cos(s);
	T_SET[1*NTREF+1]=SG1*cos(s);
	T_SET[2*NTREF+1]=SH1*sin(s);
	
	N_SET[0*NTREF+2]= DF1*sin(s);
	N_SET[1*NTREF+2]=-DG1*cos(s);
	T_SET[0*NTREF+2]= SF1*sin(s);
	T_SET[1*NTREF+2]= SG1*sin(s);
	T_SET[2*NTREF+2]=-SH1*cos(s);
	
	NK[0]=0;
	NK[1]=1;
	NK[2]=1;
	
	nt=3;
	// k>=2
	while(nt<NTREF)
	{
		k=k+1;
		
		B1=(2*(1-nu)+k*(1+nu))/((k-1)*(2*(1-nu)-k*(1+nu)))*pow(b,(2*k));
		B2=pow(k,2)*(1+nu)/((k-1)*(2*(1-nu)-k*(1+nu)))*pow(b,2);
		B3=(2*(1-nu)+k*(1+nu))/(1-k)/(1+nu)*pow(b,(-2));
		B4=1.0/(1-k)*pow(b,(-2*k));
		B5=(k*(1+nu)-4)/((k-1)*(2*(1-nu)-k*(1+nu)))*pow(b,(2*k));
		B6=(k*(1+nu)+4)/(2*(1-nu)-k*(1+nu));
		B7=(k*(1+nu)-4)/(1-k)/(1+nu)*pow(b,(-2));
		J1=  E*((1+nu-k)*B1+nu*k*B5)/(1-pow(nu,2));
		J2=  E*(k-1)*B2/(1+nu);
		J3=  E*((1+nu+k)+nu*k*B6)/(1-pow(nu,2));
		J4=  E*((1+nu-nu*k)*B1+k*B5)/(1-pow(nu,2));
		J5=  E*((1+nu+nu*k)+k*B6)/(1-pow(nu,2));
		J6= -E*k*(B1+B5)/2.0/(1+nu);
		J7=  E*k*(B6-1)/2.0/(1+nu);
		J8=  E*((1+nu-k)*B3+nu*k*B7)/(1-pow(nu,2));
		J9=  E*(k-1)*B4/(1+nu);
		J10=-E*(1+k)/(1+nu);
		J11= E*((1+nu-nu*k)*B3+k*B7)/(1-pow(nu,2));
		J12=-E*k*(B3+B7)/2.0/(1+nu);
		
		DF2=B1*pow(r,(1-k))+B2*pow(r,(k-1))+pow(r,(1+k));
		DG2=B5*pow(r,(1-k))-B2*pow(r,(k-1))+B6*pow(r,(1+k));
		DF3=B3*pow(r,(1-k))+B4*pow(r,(k-1))+pow(r,(-1-k));
		DG3=B7*pow(r,(1-k))-B4*pow(r,(k-1))+pow(r,(-1-k));
		SF2=J1* pow(r,(-k))+J2*pow(r,(k-2))+J3*pow(r,(k));
		SG2=J4* pow(r,(-k))-J2*pow(r,(k-2))+J5*pow(r,(k));
		SH2=J6* pow(r,(-k))-J2*pow(r,(k-2))+J7*pow(r,(k));
		SF3=J8* pow(r,(-k))+J9*pow(r,(k-2))+J10*pow(r,(-2-k));
		SG3=J11*pow(r,(-k))-J9*pow(r,(k-2))-J10*pow(r,(-2-k));
		SH3=J12*pow(r,(-k))-J9*pow(r,(k-2))+J10*pow(r,(-2-k));
		
		N_SET[0*NTREF+nt]=DF2*cos(k*s);
		N_SET[1*NTREF+nt]=DG2*sin(k*s);
		T_SET[0*NTREF+nt]=SF2*cos(k*s);
		T_SET[1*NTREF+nt]=SG2*cos(k*s);
		T_SET[2*NTREF+nt]=SH2*sin(k*s);
		NK[nt]=k;
		
		nt=nt+1;
		N_SET[0*NTREF+nt]= DF2*sin(k*s);
		N_SET[1*NTREF+nt]=-DG2*cos(k*s);
		T_SET[0*NTREF+nt]= SF2*sin(k*s);
		T_SET[1*NTREF+nt]= SG2*sin(k*s);
		T_SET[2*NTREF+nt]=-SH2*cos(k*s);
		NK[nt]=k;
		
		nt=nt+1;
		if(nt>=NTREF)
		{
			break;
		}
		N_SET[0*NTREF+nt]=DF3*cos(k*s);
		N_SET[1*NTREF+nt]=DG3*sin(k*s);
		T_SET[0*NTREF+nt]=SF3*cos(k*s);
		T_SET[1*NTREF+nt]=SG3*cos(k*s);
		T_SET[2*NTREF+nt]=SH3*sin(k*s);
		NK[nt]=k;
		
		nt=nt+1;
		N_SET[0*NTREF+nt]= DF3*sin(k*s);
		N_SET[1*NTREF+nt]=-DG3*cos(k*s);
		T_SET[0*NTREF+nt]= SF3*sin(k*s);
		T_SET[1*NTREF+nt]= SG3*sin(k*s);
		T_SET[2*NTREF+nt]=-SH3*cos(k*s);
		NK[nt]=k;
		
		nt=nt+1;
	}
	// Transformation of coordinates
	DM=(double *)calloc(2*2,sizeof(double));
	DUCLEAN(2,2,DM);
	DM[0*2+0]=cos(s); DM[0*2+1]=-sin(s);
	DM[1*2+0]=sin(s); DM[1*2+1]= cos(s);

	SM=(double *)calloc(3*3,sizeof(double));
	DUCLEAN(3,3,SM);
	SM[0*3+0]=pow(cos(s),2); SM[0*3+1]=pow(sin(s),2); 
	SM[0*3+2]=-sin(2*s);
	SM[1*3+0]=pow(sin(s),2); SM[1*3+1]=pow(cos(s),2); 
	SM[1*3+2]= sin(2*s);
	SM[2*3+0]=0.5*sin(2*s); SM[2*3+1]=-0.5*sin(2*s); 
	SM[2*3+2]=cos(2*s);

	TEMP=(double *)calloc(2*NTREF,sizeof(double));
	DUCLEAN(2,NTREF,TEMP);
	MATMULT(DM,N_SET,2,2,NTREF,TEMP);
	for(i=0;i<2;i++)
	{
		for(j=0;j<NTREF;j++)
		{
			N_SET[i*NTREF+j]=TEMP[i*NTREF+j];
		}
	}
	TEMP=(double *)calloc(3*NTREF,sizeof(double));
	DUCLEAN(3,NTREF,TEMP);
	MATMULT(SM,T_SET,3,3,NTREF,TEMP);
	for(i=0;i<3;i++)
	{
		for(j=0;j<NTREF;j++)
		{
			T_SET[i*NTREF+j]=TEMP[i*NTREF+j];
		}
	}
	free(TEMP); free(DM); free(SM); free(NK);
	return;
}
