/*
*******************************************************
* Subroutine TREFFTZ                                  *
* - Evaluate Trefftz functions truncted with          * 
*   specified terms number                            *
*******************************************************
*/
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
void TREFFTZ(double sp,double tp,int kMATS,
			 double PROPS[],double N_SET[],
			 double T_SET[])
{
	void DUCLEAN();
	void ITCLEAN();
	void ocpow();
	void ocmul();
	extern int NTREF,NTYPE,NNODE,NDOFN,NPROP;
	int temp,im,km,nt,k,n1,n2,*NK;
	double u1,v1,u2,v2,u3,v3,u4,v4,u5,v5;
	double YOUNG,POISS,KAMA,GMOD,*RZK,*RRK,*RSK,*IZK,*IRK,*ISK;
	// Check the number of terms of Trefftz functions
	temp=NNODE*NDOFN-3;
	if(NTREF<temp)
	{
		printf("Small number of Trefftz functions!");
		exit(0);
	}
	//
	YOUNG=PROPS[kMATS*NPROP+0];
	POISS=PROPS[kMATS*NPROP+1];
	if(NTYPE==1) // plane stress
	{
		KAMA=(3.0-POISS)/(1.0+POISS);
	}
	else if(NTYPE==2) // plane strain
	{
		KAMA=3.0-4*POISS;
	}
	GMOD=YOUNG/(2.0*(1+POISS));
	
	RZK=(double *)calloc(NTREF,sizeof(double));
	RRK=(double *)calloc(NTREF,sizeof(double));
	RSK=(double *)calloc(NTREF,sizeof(double));
	DUCLEAN(1,NTREF,RZK);
	DUCLEAN(1,NTREF,RRK);
	DUCLEAN(1,NTREF,RSK);
    IZK=(double *)calloc(NTREF,sizeof(double));
	IRK=(double *)calloc(NTREF,sizeof(double));
	ISK=(double *)calloc(NTREF,sizeof(double));
	DUCLEAN(1,NTREF,IZK);
	DUCLEAN(1,NTREF,IRK);
	DUCLEAN(1,NTREF,ISK);

	k=1;
	ocmul(KAMA-1,0.0,sp,tp,&RZK[0],&IZK[0]);
	ocmul( 0.0,1.0,sp,-tp,&RZK[1],&IZK[1]);
	ocmul(-1.0,0.0,sp,-tp,&RZK[2],&IZK[2]);

	RRK[0]=2.0; IRK[0]=0.0;
	RRK[1]=0.0; IRK[1]=0.0;
	RRK[2]=0.0; IRK[2]=0.0;

	RSK[0]=0.0; ISK[0]=0.0;
	RSK[1]=0.0; ISK[1]=1.0;
	RSK[2]=1.0; ISK[2]=0.0;

	NK=(int *)calloc(NTREF,sizeof(int));
	ITCLEAN(1,NTREF,NK);
	NK[0]=k; NK[1]=k; NK[2]=k;
	nt=3;
	while(nt<NTREF)
	{
		k=k+1;

		ocpow(sp,tp,k,&u1,&v1);
		ocmul(0.0,KAMA,u1,v1,&u2,&v2);
		ocmul(0.0,1.0*k,sp,tp,&u3,&v3);
		ocpow(sp,-tp,k-1,&u4,&v4);
		ocmul(u3,v3,u4,v4,&u5,&v5);
		RZK[nt]=u2+u5; IZK[nt]=v2+v5;

		ocpow(sp,tp,k-1,&u1,&v1);
		ocmul(0.0,2.0*k,u1,v1,&u2,&v2);
		RRK[nt]=u2; IRK[nt]=v2;

		ocmul(0.0,1.0*k*(k-1),sp,-tp,&u1,&v1);
		ocpow(sp,tp,k-2,&u2,&v2);
		ocmul(u1,v1,u2,v2,&u3,&v3);
		RSK[nt]=u3; ISK[nt]=v3;
		NK[nt]=k;

		nt=nt+1;
		ocpow(sp,tp,k,&u1,&v1);
		ocpow(sp,-tp,k-1,&u2,&v2);
		ocmul(sp,tp,u2,v2,&u3,&v3);
		RZK[nt]=KAMA*u1-k*u3; IZK[nt]=KAMA*v1-k*v3;

		ocpow(sp,tp,k-1,&u1,&v1);
		RRK[nt]=2*k*u1; IRK[nt]=2*k*v1;

		ocpow(sp,tp,k-2,&u1,&v1);
		ocmul(u1,v1,sp,-tp,&u2,&v2);
		RSK[nt]=k*(k-1)*u2; ISK[nt]=k*(k-1)*v2;
		NK[nt]=k;

		nt=nt+1;
		if(nt>=NTREF)
		{
			break;
		}
		ocpow(sp,-tp,k,&u1,&v1);
		ocmul(0.0,1.0,u1,v1,&u2,&v2);
		RZK[nt]=u2; IZK[nt]=v2;

		RRK[nt]=0.0; IRK[nt]=0.0;

		ocpow(sp,tp,k-1,&u1,&v1);
		ocmul(0.0,k*1.0,u1,v1,&u2,&v2);
		RSK[nt]=u2; ISK[nt]=v2;
		NK[nt]=k;

		nt=nt+1;
		ocpow(sp,-tp,k,&u1,&v1);
		RZK[nt]=-u1; IZK[nt]=-v1;

		RRK[nt]=0.0; IRK[nt]=0.0;

		ocpow(sp,tp,k-1,&u1,&v1);
		RSK[nt]=k*u1; ISK[nt]=k*v1;
		NK[nt]=k;

		nt=nt+1;
	}
	for(im=0;im<NTREF;im++)
	{
		N_SET[0*NTREF+im]=RZK[im]/(2.0*GMOD);
		N_SET[1*NTREF+im]=IZK[im]/(2.0*GMOD);
		k=NK[im];
		n1=4*(k-2)+0;
		n2=4*(k-2)+1;
		km=im-3;
		if((im==0)||(km==n1)||(km==n2))
		{
			T_SET[0*NTREF+im]=RRK[im]-RSK[im];
			T_SET[1*NTREF+im]=RRK[im]+RSK[im];
			T_SET[2*NTREF+im]=ISK[im];
		}
		else
		{
			T_SET[0*NTREF+im]=-RSK[im];
			T_SET[1*NTREF+im]= RSK[im];
			T_SET[2*NTREF+im]= ISK[im];
		}
	}
	return;
}
