function [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,kTSEL,...
    CLENG,PROPS)
% Compute element matrix G
%   G = integral( QT* ShapEge) along element boundary
% Input parameters:
%   ECOOD: Coordinates of nodes of specified element
%   ELNOD: Local relation of edge and nodes
%   kMATS: Material number of specified element
%   kTSEL: Element type
%   CLENG: Element characteristic length
%   PROPS: Properties of materials
% Output parameters:
%   EGMTX: Element G matrix
% *************************************************
global NTREF NDIME NDOFN NEDGE NNODE NGAUS;

% Gaussian point and weight coefficients
[POSGP,WEIGP]=GAUSSQU(NGAUS);
% Material properties
THICK=PROPS(kMATS,3);

% Compute G matrix of every element
EGMTX=zeros(NTREF,NNODE);
for iEDGE=1:NEDGE
    for iGAUS=1:NGAUS
        EXISP=POSGP(iGAUS);
        [CORGS,DVOLU,AMTRX,SHMTX]=QUANGAS(iEDGE,...
            EXISP,ECOOD,ELNOD);
        DVOLU=DVOLU*WEIGP(iGAUS);
        if THICK+1~=1
            DVOLU=DVOLU*THICK;
        end
        xp=CORGS(1);
        yp=CORGS(2);
        if kTSEL==0 % general element
            [N_SET,T_SET]=TREFFTZ(xp,yp);
        elseif kTSEL==1 % special circular hole element
            [N_SET,T_SET]=SCHTREF(xp,yp,CLENG);
        end
        Q_SET=AMTRX*T_SET;
        QTShapEge=Q_SET'*SHMTX;
        for im=1:NTREF
            for jn=1:NNODE
                EGMTX(im,jn)=EGMTX(im,jn)+...
                    QTShapEge(im,jn)*DVOLU;
            end
        end
    end
end
clear Q_SET QTShapEge N_SET T_SET POSGP WEIGP SHMTX;