function [c0]=RigidRecover(ECOOD,c_Ele,d_Ele,...
    kTSEL,CLENG)
% Recovery of rigid body motion
% Input parameters:
%   ECOOD: Coordinates of element nodes
%   c_Ele: Coefficients of Trefftz interpolation
%   d_Ele: Displacement field at element nodes
%   kTSEL: Element type
%   CLENG: Element characteristic length
% Output parameters:
%   c0: Rigid body motion term
% **********************************

global NNODE;
sum=0;
for iNODE=1:NNODE
    xp=ECOOD(iNODE,1);
    yp=ECOOD(iNODE,2);
    if kTSEL==0 % general element
        [N_SET,T_SET]=TREFFTZ(xp,yp);
    elseif kTSEL==1 % circular hole element
        [N_SET,T_SET]=SCHTREF(xp,yp,CLENG);
    end
    sum=sum+(d_Ele(iNODE)-N_SET*c_Ele);
end
c0=sum/NNODE;