function [N_SET,T_SET]=SCHTREF(sp,tp,b)
% Compute the special circular hole Trefftz functions
% with specified terms
% Input parameters:
%   sp, tp: coordinates
%   b     : radius of circular hole
% Output parameters:
%   N_SET: Trefftz functions
%   T_SET: Derivatives of Trefftz functions
%              dN/ds and dN/dt
% ************************************************
global NTREF NNODE NDOFN;

% Check the number of terms of Trefftz functions
temp=NNODE*NDOFN-1;
if NTREF<temp
    error('Too small terms of Trefftz functions!');
end
% Compute N_SET and T_SET
N_SET=zeros(1,NTREF);
T_SET=zeros(2,NTREF);

r=sqrt(sp^2+tp^2);
s=atan2(tp,sp);

for im=1:NTREF
    % Determine the order of Ni
    n=ceil(im/2);
    % Justify im is odd or even number
    remaind=rem(im,2);
    if 1+remaind==1 % im is even
        N_SET(1,im)=(r^n+b^(2*n)*r^(-n))*sin(n*s);
        T_SET(1,im)=r^(n-1)*n*sin(n*s-s)-...
            b^(2*n)*r^(-n-1)*n*sin(n*s+s);
        T_SET(2,im)=r^(n-1)*n*cos(n*s-s)+...
            b^(2*n)*r^(-n-1)*n*cos(n*s+s);
    else % im is odd, that is, im=2*n-1
        N_SET(1,im)=(r^n+b^(2*n)*r^(-n))*cos(n*s);
        T_SET(1,im)= r^(n-1)*n*cos(n*s-s)-...
            b^(2*n)*r^(-n-1)*n*cos(n*s+s);
        T_SET(2,im)=-r^(n-1)*n*sin(n*s-s)-...
            b^(2*n)*r^(-n-1)*n*sin(n*s+s);
    end
end