function [GSTIF]=ASMSTIF(iELEM,LNODS,ESTIF,GSTIF)
% ** Assemble element stiffness to global stiffness
% Input parameters:
%   iELEM: Element index
%   LNODS: Elementary connectivity
%   ESTIF: Element stiffness
%   GSTIF: Global stiffness
% Output parameters:
%   GSTIF: Global stiffness
% Note: 
%	1. Assemble process can be used to other problems
%	directly
%   2. Full matrix is stored instead of bandwidth form
% **********************************
global NDOFN NNODE;

for iNODE=1:NNODE
    iPOIN=LNODS(iELEM,iNODE);
    for iDOFN=1:NDOFN
        %global row location
        iGR=NDOFN*(iPOIN-1)+iDOFN;
        %local row location
        iLR=NDOFN*(iNODE-1)+iDOFN; 
        for jNODE=1:NNODE
            jPOIN=LNODS(iELEM,jNODE);
            for jDOFN=1:NDOFN
                % global column location
                jGC=NDOFN*(jPOIN-1)+jDOFN; 
                % local column location
                jLC=NDOFN*(jNODE-1)+jDOFN; 
                GSTIF(iGR,jGC)=GSTIF(iGR,jGC)+...
                    ESTIF(iLR,jLC);
            end
        end
    end
end
clear ESTIF;