function [GSTIF,GLOAD]=INDISBC(NEQNS,NVFIX,NOFIX,...
    IFPRE,PRESC,GSTIF,GLOAD)
% ** Modify global stiffness matrix GSTIF and
%    equivalent load vector GLOAD by the penalty
%    approach
% Input parameters:
%   NEQNS: Total number of equations
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted
%   NOFIX: Global index of nodes at which specified DOF
%          is restricted
%   IFPRE: Types of constraints of each DOF
%   PRESC: Specified values
%   GSTIF: Global stiffness matrix
%   GLOAD: Global equivalent nodal load vector
% Output parameters:
%   GSTIF: Modified global stiffness matrix
%   GLOAD: Modified global equivalent nodal load vector
% **********************************
global NDOFN;

% Decide penalty parameter CNST
CNST=max(max(abs(GSTIF)));
if CNST+1==1
    error('**Singular coefficient matrix GSTIF');
end
CNST=CNST*1000000;
% Modify GSTIF and GLOAD for specified nodal
% displacements
for iVFIX=1:NVFIX
    kPOIN=NOFIX(iVFIX);
    for iDOFN=1:NDOFN
        iGR=(kPOIN-1)*NDOFN+iDOFN;
        ii=IFPRE(iVFIX,iDOFN);
        % 1: a constrained DOF
        % 0: no constrained DOF
        if ii==1
            disv=PRESC(iVFIX,iDOFN);
            GSTIF(iGR,iGR)=GSTIF(iGR,iGR)+CNST;
            GLOAD(iGR)=GLOAD(iGR)+CNST*disv;
        end
    end
end