function [x]=LSSOLVR(A,b,n)
% ** Solve Linear system of equations Ax=b
% Input parameters:
%   A: coefficient matrix
%   b: right-handed vector
%   n: the number of equations
% Output parameters:
%   x: solution of Ax=b
% **********************************

% Equilibration treatment
[A,b]=EQUIL(A,b,n);
% Condition number
condnum=cond(A);
% MATLAB Command
x=A\b;
% Residual norm
resnorm=norm(A*x-b);
disp(['Condition number = ',num2str(condnum)]);
disp(['Residual  norm   = ',num2str(resnorm)]);


% --------------------------------------------
function [A,b]=EQUIL(A,b,n)
% Equilibration treatment of Ax=b
% **********************************
% Determine maximum elements in each row and store them
% in vector temp
temp=(max(abs(A')))';
if min(temp)+1==1
    error('**Warning: Matrix is singular, check it!');
end
% Elements in each row divide the maximum element
for ii=1:n
    A(ii,:)=A(ii,:)/temp(ii);
end
b=b./temp;