function [CORGS,DVOLU,AMTRX,SHMTX]=...
    QUANGAS(iEDGE,EXISP,ECOOD,ELNOD)
% Evaluate quantities at Gaussian points
% Input parameters:
%   iEDGE: Number index of element edge
%   EXISP: Local coordinate of Gaussian points
%   ECOOD: Coordinates of element nodes
%   ELNOD: Local relations of edge and nodes
% Output parameters:
%   CORGS: Coordinates of Gauss point
%   DVOLU: Functions of coordinate tranformation for
%          Gauss integral 
%   AMTRX: Directional cosine at Gauss point
%   SHMTX: Shape function of frame field defined on
%          entire boundary 
% *************************************

global NDIME NDOFN NNODE NEDGE NODEG;

% Shape functions and its derivatives for 1D line
% element
SHAPE=zeros(1,NODEG);
DSHAP=zeros(1,NODEG);
[SHAPE,DSHAP]=SHAPFUN(EXISP);

% Coordinates and derivatives of Gauss points 
%  x=sum(Ni*xi) and y=sum(Ni*yi)
%  dx/dt=sum(dNi/dt*xi) and dy/dt=sum(dNi/dt*yi)
CORGS=zeros(1,NDIME);
DERGS=zeros(1,NDIME);
for iDIME=1:NDIME
    for iODEG=1:NODEG
        kNODE=ELNOD(iEDGE,iODEG);
        CORGS(iDIME)=CORGS(iDIME)+...
            SHAPE(iODEG)*ECOOD(kNODE,iDIME);
        DERGS(iDIME)=DERGS(iDIME)+...
            DSHAP(iODEG)*ECOOD(kNODE,iDIME);
    end
end
% Coordinate tranformation for Gauss integral
%  DVOLU=sqrt((dx/dt)^2+(dy/dt)^2)
DVOLU=sqrt(DERGS(1)^2+DERGS(2)^2);

% Directional cosine at Gauss point
%  nx = dy/dS   ny = - dx/dS
AMTRX=zeros(1,NDIME);
AMTRX(1)= DERGS(2)/DVOLU;
AMTRX(2)=-DERGS(1)/DVOLU;

% Form shape function matrix of frame function
SHMTX=zeros(1,NNODE);
iNODE=ELNOD(iEDGE,1); % start node of the edge
for iODEG=1:NODEG
    kNODE=iNODE+iODEG-1;
    if kNODE>NNODE
        kNODE=1;
    end
    SHMTX(kNODE)=SHAPE(iODEG);
end