function [EHMTX]=HMATRIX(ECOOD,ELNOD,kMATS,PROPS)
% Compute element matrix H
%   H = integral( QT * N) on element boundary
% Input parameters:
%   ECOOD: Coordinates of nodes of specified element
%   ELNOD: Local relation of edge and nodes
%   kMATS: Material number of specified element
%   PROPS: Properties of materials
% Output parameters:
%   EHMTX: Element H matrix
% ***********************************
global NTREF NDIME NDOFN NNODE NEDGE NGAUS;

% Gaussian point and weight coefficients
[POSGP,WEIGP]=GAUSSQU(NGAUS);
% Material properties
THICK=PROPS(kMATS,3);
% Compute H matrix of every edge of element
EHMTX=zeros(NTREF,NTREF);
for iEDGE=1:NEDGE
    for iGAUS=1:NGAUS
        EXISP=POSGP(iGAUS);
        [CORGS,DVOLU,AMTRX,SHMTX]=QUANGAS(iEDGE,...
            EXISP,ECOOD,ELNOD);
        DVOLU=DVOLU*WEIGP(iGAUS);
        if THICK+1~=1
            DVOLU=DVOLU*THICK;
        end
        [N_SET,T_SET]=TREFFTZ(CORGS(1),CORGS(2));
        Q_SET=AMTRX*T_SET;
        QTN=Q_SET'*N_SET; % m by m
        for im=1:NTREF
            for jm=1:NTREF
                EHMTX(im,jm)=EHMTX(im,jm)+...
                    QTN(im,jm)*DVOLU;
            end
        end
    end
end
clear Q_SET QTN N_SET T_SET POSGP WEIGP SHMTX AMTRX;