function [GSTIF,GLOAD]=INDISBC(COORD,NEQNS,NVFIX,...
    NOFIX,IFPRE,PRESC,GSTIF,GLOAD,NINTP,IPCOD,CRBFI)
% Modify global stiffness matrix GSTIF and equivalent
% force vector GLOAD by the penalty approach
% Input parameters:
%   COORD: Coordinates of nodes
%   NEQNS: Total number of equations
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted 
%   NOFIX: Global index of nodes at which specified DOF
%          is restricted 
%   IFPRE: Types of constraints of each DOF
%   PRESC: Specified values
%   GSTIF: Global stiffness matrix
%   GLOAD: Global equivalent nodal load vector
%   NINTP: Number of RBF interpolation points
%   IPCOD: Coordinates of RBF interpolation points
%   CRBFI: Coefficient of RBF interpolation
% Output parameters:
%   GSTIF: Modified global stiffness matrix
%   GLOAD: Modified global equivalent nodal load vector
% **********************************
global NDOFN;

if NVFIX>0
    % Decide penalty parameter CNST
    CNST=max(max(abs(GSTIF)));
    if CNST+1==1
        error('**Singular stiffness matrix GSTIF!');
    end
    CNST=CNST*1000000;
    % Modify GSTIF and GLOAD for specified nodal
    % displacements
    for iVFIX=1:NVFIX
        kPOIN=NOFIX(iVFIX);
        xp=COORD(kPOIN,1);
        yp=COORD(kPOIN,2);
        for iDOFN=1:NDOFN
            iGR=(kPOIN-1)*NDOFN+iDOFN;
            ii=IFPRE(iVFIX,iDOFN);
            % ii=1 indicates a constrained degree of
            % freedom 
            if ii==1
                disv=PRESC(iVFIX,iDOFN);
                [ups,sps]=PARSOLU(xp,yp,NINTP,...
                    IPCOD,CRBFI);
                % Modify generalized displacement BC
                mdisv=disv-ups(iDOFN);
                GSTIF(iGR,iGR)=GSTIF(iGR,iGR)+CNST;
                GLOAD(iGR)=GLOAD(iGR)+CNST*mdisv;
            end
        end
    end
end