function [NPOIN,NELEM,COORD,MATNO,LNODS,NVFIX,NOFIX,...
    IFPRE,PRESC,NPLOD,NDLEG,LODPT,POINT,NEASS,NOPRS,...
    PRESS,PROPS]=INPUTDT
% Input data from a file
% Input parameters: No
% Output parameters:
%   NPOIN: Number of nodes in domain
%   NELEM: Number of elements in domain
%   COORD: Coordinates of nodes
%   MATNO: Material index of each element
%   LNODS: Element connectivity
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted 
%   NOFIX: Global index of nodes at which specified DOF
%          is restricted 
%   IFPRE: Types of constraints of each DOF
%   PRESC: Specified values
%   NPLOD: Number of concentrated loads
%   NDLEG: Number of loaded edges
%   LODPT: Global index of nodes at which concentrated
%          loads are applied 
%   POINT: Specified values of concentrated loads
%   NEASS: Element index with loaded edge
%   NOPRS: Global node index along loaded edge
%   PRESS: Specified values of distributed loads at
%          nodes 
%   PROPS: Properties of materials
% **********************************
global NTREF NTYPE NDIME NDOFN NSTRE NNODE NODEG NEDGE;
global NMATS NPROP NGAUS;

% Open the input file
FILE1=input('Input data file name: ','s');
fp=fopen(FILE1,'r');
if fp<0
    error('-- Error: Cant open data file!');
end
%fp=fopen('input.txt','r');

dummy=char(zeros(1,100));  % fill with ASCII zeros
TITLE=char(zeros(1,200));  % fill with ASCII zeros

% Description of problem
dummy = fgets(fp);
TITLE = fgets(fp);
dummy = fgets(fp);
% Number of m-set and type definition
dummy = fgets(fp);
TMP = str2num(fgets(fp));
[NTREF,NTYPE]=deal(TMP(1),TMP(2));
% Element properties
dummy = fgets(fp);
TMP = str2num(fgets(fp));
[NNODE,NEDGE,NODEG]=deal(TMP(1),TMP(2),TMP(3));
% Number of Dimensions, DOF and stress components
dummy = fgets(fp);
TMP = str2num(fgets(fp));
[NDIME,NDOFN,NSTRE]=deal(TMP(1),TMP(2),TMP(3));
% Number of materials, material properties and Gaussian
% points
dummy = fgets(fp);
TMP = str2num(fgets(fp));
[NMATS,NPROP,NGAUS]=deal(TMP(1),TMP(2),TMP(3));
% Number of nodes, elements and boundary conditions
dummy = fgets(fp);
TMP = str2num(fgets(fp));
[NPOIN,NELEM,NVFIX,NPLOD,NDLEG]=...
    deal(TMP(1),TMP(2),TMP(3),TMP(4),TMP(5));

% Read element connectivity and material numbers
MATNO=zeros(NELEM,1);
LNODS=zeros(NELEM,NNODE);
dummy = fgets(fp);
dummy = fgets(fp);
for iELEM=1:NELEM
	TMP=str2num(fgets(fp));
    [N,MATNO(iELEM),LNODS(iELEM,:)]=...
        deal(TMP(1),TMP(2),TMP(3:2+NNODE));
end

% Read nodal coordinates
COORD=zeros(NPOIN,NDIME);
dummy = fgets(fp);
dummy = fgets(fp);
for iPOIN=1:NPOIN
    TMP=str2num(fgets(fp));
    [N,COORD(iPOIN,:)]=deal(TMP(1),TMP(2:1+NDIME));
end
% Read essential boundary conditions
NOFIX=zeros(NVFIX,1);
IFPRE=zeros(NVFIX,NDOFN);
PRESC=zeros(NVFIX,NDOFN);
dummy = fgets(fp);
dummy = fgets(fp);
for iVFIX=1:NVFIX
    TMP=str2num(fgets(fp));
    [N,NOFIX(iVFIX),IFPRE(iVFIX,:),PRESC(iVFIX,:)]=...
        deal(TMP(1),TMP(2),TMP(3:2+NDOFN),...
        TMP(3+NDOFN:2+2*NDOFN));
end
% Read natural boundary conditions
LODPT=zeros(NPLOD,1);
POINT=zeros(NPLOD,NDOFN);
NEASS=zeros(NDLEG,1);
NOPRS=zeros(NDLEG,NODEG);
PRESS=zeros(NDLEG,NODEG*NDOFN);
% Read concentrated loads
dummy = fgets(fp);
if NPLOD>0
    dummy = fgets(fp);
    for iPLOD=1:NPLOD
        TMP=str2num(fgets(fp));
        [N,LODPT(iPLOD),POINT(iPLOD,:)]=...
            deal(TMP(1),TMP(2),TMP(3:2+NDOFN));
    end
end
% Read distributed edge loads
dummy = fgets(fp);
if NDLEG>0
    dummy = fgets(fp);
    for iDLEG=1: NDLEG
        TMP=str2num(fgets(fp));
        [N,NEASS(iDLEG),NOPRS(iDLEG,:),PRESS(iDLEG,:)]...
            =deal(TMP(1),TMP(2),TMP(3:2+NODEG),...
            TMP(3+NODEG:2+NODEG+NODEG*NDOFN));        
    end
end
% Read material properties
PROPS=zeros(NMATS,NPROP);
dummy = fgets(fp);
dummy = fgets(fp);
for iMATS=1:NMATS
    TMP=str2num(fgets(fp));
    [N,PROPS(iMATS,:)]=deal(TMP(1),TMP(2:1+NPROP));
end
fclose(fp);