function MAINFUN
% Main program using HTFEM with RBF approximation for
% 2D Poisson problems
% *******************************
global NTREF NTYPE NDIME NDOFN NNODE NEDGE NODEG NSTRE;
global NMATS NPROP NGAUS;

disp('**********************************************');
disp('             Hybrid Trefftz FEM');
disp('          for 2D Poisson problems');
disp('**********************************************');

% Input data from file
[NPOIN,NELEM,COORD,MATNO,LNODS,NVFIX,NOFIX,IFPRE,...
    PRESC,NPLOD,NDLEG,LODPT,POINT,NEASS,NOPRS,PRESS,...
    PROPS]=INPUTDT;
% Generate local relations of nodes and edges
[ELNOD]=TYPELEM;
% Compute the coefficient of RBF interpolation
[NINTP,IPCOD,CRBFI]=RBFINTP(NPOIN,NELEM,COORD,LNODS);
% Global stiffness matrix
NEQNS=NPOIN*NDOFN;
GSTIF=zeros(NEQNS,NEQNS);
GLOAD=zeros(NEQNS,1);
for iELEM=1:NELEM
    kMATS=MATNO(iELEM);
    % Compute some quantities related to each element
    [ECOOD,CenCoord]=ELEPARS(iELEM,LNODS,COORD);
    % Compute H matrix
    [EHMTX]=HMATRIX(ECOOD,ELNOD,kMATS,PROPS);
    % Compute G matrix
    [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,PROPS);
    % Compute element stiffnesss matrix
    [ESTIF]=KMATRIX(EHMTX,EGMTX);
    % Assemble stiffness matrix
    [GSTIF]=ASMSTIF(iELEM,LNODS,ESTIF,GSTIF);
end
% Compute equivalent nodal forces
[GLOAD]=PVECTOR(MATNO,PROPS,LNODS,COORD,NDLEG,NEASS,...
    NOPRS,PRESS,NPLOD,LODPT,POINT,GLOAD,NINTP,IPCOD,...
    CRBFI);
% Introduce constrained displacements and point loads
[GSTIF,GLOAD]=INDISBC(COORD,NEQNS,NVFIX,NOFIX,IFPRE,...
    PRESC,GSTIF,GLOAD,NINTP,IPCOD,CRBFI);
% Solve linear system of equations and store
% displacements of each node
% in the array ASDIS
[ASDIS]=LSSOLVR(GSTIF,GLOAD,NEQNS);
% Output nodal displacements
[UPOIN]=FIEDNOD(NPOIN,COORD,ASDIS,NINTP,IPCOD,CRBFI);
% Compute displacement and stress components at
% central point of element
[CECOD,UCENP,SCENP]=FIEDCEN(NELEM,MATNO,LNODS,COORD,...
    PROPS,ELNOD,ASDIS,NINTP,IPCOD,CRBFI);
%-- Output results
OPRESUT(NPOIN,COORD,UPOIN,NELEM,CECOD,UCENP,SCENP,...
    NVFIX,NPLOD,NDLEG);
disp('--- All procedure are finished ---');