function [NINTP,IPCOD,CRBFI]=RBFINTP(NPOIN,NELEM,COORD,LNODS)
% RBF interpolation for source field in the domain
% Input parameters:
%   NELEM:  Number of elements
%   NPOIN:  Number of nodes in the domain
%   COORD:  Coordinates of nodes
%   LNODS:  Element connectivity
% Output parameters:
%   NINTP: Number of interpolation points
%   IPCOD: Coordinates of interpolation points
%   CRBFI: Coefficient of RBF interpolation
% ************************************************
global NDIME NDOFN NNODE;

% Generate interpolation points: node+centroid
NINTP=NPOIN+NELEM;
IPCOD=zeros(NINTP,NDIME);
for iPOIN=1:NPOIN
    IPCOD(iPOIN,:)=COORD(iPOIN,:);
end
for iELEM=1:NELEM
    sum0=zeros(1,NDIME);
    for iNODE=1:NNODE
        kPOIN=LNODS(iELEM,iNODE);
        sum0(1,:)=sum0(1,:)+COORD(kPOIN,:);
    end
    IPCOD(NPOIN+iELEM,:)=sum0(1,:)/NNODE;
end

NVARB=NINTP*NDOFN;
FMATX=zeros(NVARB,NVARB);
bvect=zeros(NVARB,1);
for iINTP=1:NINTP
    xi=IPCOD(iINTP,1);
    yi=IPCOD(iINTP,2);
    % User function is used to provide the internal
    % source distribution 
    [bvect(iINTP)]=USERFUN(xi,yi);
    for jINTP=1:NINTP
        xj=IPCOD(jINTP,1);
        yj=IPCOD(jINTP,2);
        r=sqrt((xi-xj)^2+(yi-yj)^2);
        % PS: r^(2*n-1)
        n=2;
        FMATX(iINTP,jINTP)=r^(2*n-1);
    end
end
% Solve linear algebraic equations
[CRBFI]=LSSOLVR(FMATX,bvect,NVARB);