function [GBFOR,TEMPR]=USERFUN(x,y,PROPS)
% Compute the source value at given point (x,y)
% Input parameters:
%   x, y :  Coordinates of computing point
%   PROPS:  Material properties
% Output parameters:
%   GBFOR:  generalized body forces at point (x,y)
%   TEMPR:  Temperature at point (x,y)
% ************************************************
global NTYPE;

% Material properties
YOUNG=PROPS(1,1);
POISS=PROPS(1,2);
THICK=PROPS(1,3);
TEXPN=PROPS(1,4);
DENST=PROPS(1,5);
%
if NTYPE==1 %plane stress
    gm=YOUNG*TEXPN/(1-POISS);
elseif NTYPE==2
    gm=YOUNG*TEXPN/(1-2*POISS);
end
    
% Body forces
BODYF(1)=0; % bx
BODYF(2)=0; % by
% Temperature change
r=sqrt(x^2+y^2);
TEMPR=10*log(r/20)/log(5/20);% T
DTEMP(1)=10/r/log(5/20)*x/r; % dT/dx
DTEMP(2)=10/r/log(5/20)*y/r; % dT/dy
% Generalized body forces
GBFOR=zeros(2,1);
GBFOR(1)=BODYF(1)-gm*DTEMP(1);
GBFOR(2)=BODYF(2)-gm*DTEMP(2);