function [CECOD,UCENP,SCENP]=FIEDCEN(NELEM,MATNO,...
    LNODS,COORD,PROPS,ELNOD,ASDIS,NINTP,IPCOD,...
    CRBFI)
% Compute potential and flux at central point of each
% element
% Input parameters:
%   NELEM: Number of elements in domain
%   MATNO: Material index of each element
%   LNODS: Element connectivity
%   COORD: Coordinates of nodes
%   PROPS: Properties of materials
%   ELNOD: Local relation of edge and nodes
%   ASDIS: Nodal generalized displacement field in DOF
%          order
%   NINTP: Number of RBF interpolation points
%   IPCOD: Coordinates of RBF interpolation points
%   CRBFI: Coefficient of RBF interpolation
% Output parameters:
%   CECOD: Coordinates of centroid of each element
%   UCENP: Displacement fields at centroid
%   SCENP: Stress fields at centroid
% ***********************************
global NDIME NDOFN NNODE NEDGE NODEG NSTRE NTYPE NGAUS;

% Material properties
YOUNG=PROPS(1,1);
POISS=PROPS(1,2);
THICK=PROPS(1,3);
TEXPN=PROPS(1,4);
DENST=PROPS(1,5);
if NTYPE==1 %plane stress
    gm=YOUNG*TEXPN/(1-POISS);
elseif NTYPE==2
    gm=YOUNG*TEXPN/(1-2*POISS);
end

CECOD=zeros(NELEM,NDIME);
UCENP=zeros(NELEM,NDOFN);
SCENP=zeros(NELEM,NSTRE);
% Loop for all nodes
for iELEM=1:NELEM
    kMATS=MATNO(iELEM);
    % Compute some quantities related to each element
    [ECOOD,CenCoord]=ELEPARS(iELEM,LNODS,COORD);
    % Identify nodal field of the specified element
    [d_Ele]=EDISNOD(iELEM,LNODS,ASDIS);
    % Compute H matrix
    [EHMTX]=HMATRIX(ECOOD,ELNOD,kMATS,PROPS);
    % Compute G matrix
    [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,PROPS);
    % Calculate the ce coefficients: m by 1
    [c_Ele]=CMATRIX(EHMTX,EGMTX,d_Ele);
    % Recover rigid-body motion vector: 3 by 1
    [c0]=RIGIDRV(ECOOD,c_Ele,d_Ele,kMATS,PROPS);
    % Compute Trefftz internal fields at central point
    xp=0;
    yp=0;
    [N_SET,T_SET]=TREFFTZ(xp,yp,kMATS,PROPS);
    GDISP=N_SET*c_Ele;
    GSTRE=T_SET*c_Ele;
    % particular solution
    gxp=CenCoord(1)+xp;
    gyp=CenCoord(2)+yp;
    [ups,sps]=PARSOLU(gxp,gyp,NINTP,IPCOD,CRBFI,PROPS);
    % Temperature effect
    [GBFOR,TEMPR]=USERFUN(gxp,gyp,PROPS);
    % Full solution
    UCENP(iELEM,1)=GDISP(1)+c0(1)+yp*c0(3)+ups(1);
    UCENP(iELEM,2)=GDISP(2)+c0(2)-xp*c0(3)+ups(2);
    SCENP(iELEM,1)=GSTRE(1)+sps(1)-gm*TEMPR;
    SCENP(iELEM,2)=GSTRE(2)+sps(2)-gm*TEMPR;
    SCENP(iELEM,3)=GSTRE(3)+sps(3);
    % Coordinates of computing points
    CECOD(iELEM,:)=[gxp,gyp];
end
clear EHMTX EGMTX c_Ele d_Ele;