function [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,PROPS)
% Compute element matrix G
%    G = integral( QT* ShapEge) along element boundary
% Input parameters:
%   ECOOD: Coordinates of nodes of specified element
%   ELNOD: Local relation of edge and nodes
%   kMATS: Material number of specified element
%   PROPS: Properties of materials
% Output parameters:
%   EGMTX: Element G matrix
% ***********************************
global NTREF NDIME NDOFN NEDGE NNODE NGAUS;

% Gaussian point and weight coefficients
[POSGP,WEIGP]=GAUSSQU(NGAUS);
% Material properties
THICK=PROPS(kMATS,3);
NEVAB=NNODE*NDOFN;
EGMTX=zeros(NTREF,NEVAB);
for iEDGE=1:NEDGE
    for iGAUS=1:NGAUS
        EXISP=POSGP(iGAUS);
        [CORGS,DVOLU,AMTRX,SHMTX]=QUANGAS(iEDGE,...
            EXISP,ECOOD,ELNOD);
        DVOLU=DVOLU*WEIGP(iGAUS);
        if THICK+1~=1
            DVOLU=DVOLU*THICK;
        end
        [N_SET,T_SET]=TREFFTZ(CORGS(1),CORGS(2),kMATS,PROPS);
        Q_SET=AMTRX*T_SET;
        QTS=Q_SET'*SHMTX;
        for im=1:NTREF
            for jn=1:NEVAB
                EGMTX(im,jn)=EGMTX(im,jn)+...
                    QTS(im,jn)*DVOLU;
            end
        end
    end
end
clear Q_SET QTS N_SET T_SET POSGP WEIGP SHMTX AMTRX;