function [GK,GP]=INDISBC(NEQNS,PROPS,COORD,NVFIX,...
    NOFIX,IFPRE,PRESC,GK,GP,NINTP,IPCOD,CRBFI)
% Modify global stiffness matrix GSTIF and equivalent
% load vector GLOAD by the penalty approach
% Input parameters:
%   NEQNS: Number of equations
%   PROPS: Material properties
%   COORD: Coordinates of nodes
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted 
%   NOFIX: Global index of nodes at which specified DOF
%          is restricted 
%   IFPRE: Types of constraints of each DOF
%   PRESC: Specified values
%   GK   : Global stiffness matrix
%   GP   : Global equivalent nodal load vector
%   NINTP: Number of RBF interpolation points
%   IPCOD: Coordinates of RBF interpolation points
%   CRBFI: Coefficient of RBF interpolation
% Output parameters:
%   GK   : Modified global stiffness matrix
%   GP   : Modified global equivalent nodal load vector
% ********************************** 
global NDOFN;

if NVFIX>0
    % Decide penalty parameter CNST
    CNST=max(max(abs(GK)));
    if CNST+1==1
        error('**Singular stiffness matrix GK!');
    end
    CNST=CNST*1000000;
    % Modify GK and GP for specified nodal
    % displacements
    for iVFIX=1:NVFIX
        kPOIN=NOFIX(iVFIX);
        xp=COORD(kPOIN,1);
        yp=COORD(kPOIN,2);
        for iDOFN=1: NDOFN
            NR=(kPOIN-1)*NDOFN+iDOFN;
            ii=IFPRE(iVFIX,iDOFN);
            % unit value indicates a constrained DOF
            if ii==1
                disv=PRESC(iVFIX,iDOFN);
                % particular solution
                [ups,sps]=PARSOLU(xp,yp,NINTP,...
                    IPCOD,CRBFI,PROPS);
                % Evaluate homogeneous displacement BC
                mdisv=disv-ups(iDOFN);
                GK(NR,NR)=GK(NR,NR)+CNST;
                GP(NR)=GP(NR)+CNST*mdisv;
            end
        end
    end
end