function [ups,sps]=PARSOLU(xp,yp,NINTP,IPCOD,...
    CRBFI,PROPS)
% ** Evaluate approximated particular solutions at
%    given point (x,y)
% Input parameters:
%   xp,yp:  Coordinates of computing point
%   NINTP:  Number of interpolation points
%   IPCOD:  Coordinates of interpolation points
%   CRBFI:  Coefficient of RBF interpolation
%   PROPS:  Material properties
% Output parameters:
%   ups: Displacement vector
%   sps: Stress vector
% ************************************************
global NDIME NDOFN NTYPE NSTRE;

% Material properties
YOUNG=PROPS(1,1);
POISS=PROPS(1,2);
THICK=PROPS(1,3);
TEXPN=PROPS(1,4);
DENST=PROPS(1,5);
G=YOUNG/(2*(1+POISS));
if NTYPE==1 %plane stress
    mu=POISS/(1+POISS);
elseif NTYPE==2 %plane strain
    mu=POISS;
end
% Particular solutions
ups=zeros(1,NDOFN);
sps=zeros(1,NSTRE);
for jINTP=1:NINTP
    xj=IPCOD(jINTP,1);
    yj=IPCOD(jINTP,2);
    x=xp-xj;
    y=yp-yj;
    r=sqrt(x^2+y^2);
    % TPS: r^(2*n)ln(r)
    if 1+r==1
        r=0;
        lnr=0;
        r1=0;
        r2=0;
    else
        lnr=log(r);
        r1=x/r; % dr/dx
        r2=y/r; % dr/dy
    end
    n=2;
    A1=-(8*n^2+29*n+27)+8*mu*(n+2)^2+...
        2*(n+1)*(n+2)*(4*n+7-4*mu*(n+2))*lnr;
    A2=2*(n+1)*(2*n+3)-4*(n+1)^2*(n+2)*lnr;
    A3=2*n+3-2*mu*(n+2)^2+...
        2*(n+1)*(n+2)*(2*mu*n+4*mu-1)*lnr;
    A4=2*(n^2-2)-4*n*(n+1)*(n+2)*lnr;
    A5=-(2*n^2+6*n+5)+2*mu*(n+2)^2-...
        2*(n+1)*(n+2)*(-(2*n+3)+2*mu*(n+2))*lnr;
    temp1=-1/(32*G*(1-mu))*r^(2*n+2)/((n+1)^3*(n+2)^2);
    temp2=-1/(8*(1-mu))*r^(2*n+1)/((n+1)^2*(n+2)^2);
    
    u11=temp1*(A1+A2*r1*r1);
    u12=temp1*(A2*r1*r2);
    u22=temp1*(A1+A2*r2*r2);
    s111=temp2*(A3*r1+A4*r1*r1*r1+A5*(2*r1));
    s112=temp2*(A3*r2+A4*r1*r1*r2);
    s121=temp2*(A4*r1*r2*r1+A5*r2);
    s122=temp2*(A4*r1*r2*r2+A5*r1);
    s221=temp2*(A3*r1+A4*r2*r2*r1);
    s222=temp2*(A3*r2+A4*r2*r2*r2+A5*(2*r2));
    
    ups(1)=ups(1)+CRBFI(2*jINTP-1)*u11 +...
        CRBFI(2*jINTP)*u12;
    ups(2)=ups(2)+CRBFI(2*jINTP-1)*u12 +...
        CRBFI(2*jINTP)*u22;
    sps(1)=sps(1)+CRBFI(2*jINTP-1)*s111+...
        CRBFI(2*jINTP)*s112;
    sps(2)=sps(2)+CRBFI(2*jINTP-1)*s221+...
        CRBFI(2*jINTP)*s222;
    sps(3)=sps(3)+CRBFI(2*jINTP-1)*s121+...
        CRBFI(2*jINTP)*s122;
end