function [CORGS,DVOLU,AMTRX,SHMTX]=QUANGAS(iEDGE,EXISP,ECOOD,ELNOD)
% ---- Subroutine QUANGAUS is designed for Gauss integral
%   CORGS: Coordinates of Gauss point (1 by 2)
%   DVOLU: Coordinate tranformation for Gauss integral
%   AMTRX: Directional cosine matrix at Gauss point (2 by 3)
%   SHMTX: Shape function matrix of frame field on specified edge at Gauss
%   points (2 by NEVAB)

global NDIME NDOFN NNODE NEDGE NODEG;

% Shape functions and its derivatives for 1D line element
SHAPE=zeros(1,NODEG);
DSHAP=zeros(1,NODEG);
[SHAPE,DSHAP]=SHAPFUN(EXISP);

% Coordinates and derivatives of Gauss points 
%  x=sum(Ni*xi) and y=sum(Ni*yi)
%  dx/dt=sum(dNi/dt*xi) and dy/dt=sum(dNi/dt*yi)
CORGS=zeros(1,NDIME);
DERGS=zeros(1,NDIME);
for iDIME=1:NDIME
    for iODEG=1:NODEG
        kNODE=ELNOD(iEDGE,iODEG);
        CORGS(iDIME)=CORGS(iDIME)+SHAPE(iODEG)*ECOOD(kNODE,iDIME);
        DERGS(iDIME)=DERGS(iDIME)+DSHAP(iODEG)*ECOOD(kNODE,iDIME);
    end
end
% Coordinate tranformation for Gauss integral
%  DVOLU=sqrt((dx/dt)^2+(dy/dt)^2)
DVOLU=sqrt(DERGS(1)^2+DERGS(2)^2);

% Directional cosine at Gauss point
%  nx = dy/dS      ny = - dx/dS
%  A=[n1,  0, n2
%      0, n2, n1]
AMTRX=zeros(2,3);
n1= DERGS(2)/DVOLU;
n2=-DERGS(1)/DVOLU;
AMTRX(1,1)=n1;
AMTRX(1,3)=n2;
AMTRX(2,2)=n2;
AMTRX(2,3)=n1;
%AMTRX=[n1,0,n2;0,n2,n1];

% Form shape function matrix of frame function
NEVAB=NNODE*NDOFN;
SHMTX=zeros(2,NEVAB);
iNODE=ELNOD(iEDGE,1); % start node of the edge
for iODEG=1:NODEG
    kNODE=iNODE+iODEG-1;
    if kNODE>NNODE
        kNODE=1;
    end
    SHMTX(1,kNODE*2-1)=SHAPE(iODEG);
    SHMTX(2,kNODE*2)  =SHAPE(iODEG);
end