function [NINTP,IPCOD,CRBFI]=RBFINTP(NPOIN,NELEM,...
    COORD,LNODS,PROPS)
% ** RBF interpolation in the domain
% Input parameters:
%   NPOIN:  Number of nodes in the domain
%   NELEM:  Number of elements
%   COORD:  Coordinates of nodes
%   LNODS:  Element connectivity
%   PROPS:  Material properties
% Output parameters:
%   NINTP:  Number of RBF interpolation points
%   IPCOD:  Coordinates of RBF interpolation points
%   CRBFI:  Coefficients of RBF interpolation
% ************************************************
global NDIME NDOFN NNODE NTYPE;

% Material properties
YOUNG=PROPS(1,1);
POISS=PROPS(1,2);
THICK=PROPS(1,3);
TEXPN=PROPS(1,4);
DENST=PROPS(1,5);
if NTYPE==1 %plane stress
    gm=YOUNG*TEXPN/(1-POISS);
elseif NTYPE==2
    gm=YOUNG*TEXPN/(1-2*POISS);
end

% Generate interpolation points: node+centroid
NINTP=NPOIN+NELEM;
IPCOD=zeros(NINTP,NDIME);
for iPOIN=1:NPOIN
    IPCOD(iPOIN,:)=COORD(iPOIN,:);
end
% Compute central coordinates of the given element
for iELEM=1:NELEM
    sum0=zeros(1,NDIME);
    for iNODE=1:NNODE
        kPOIN=LNODS(iELEM,iNODE);
        sum0(1,:)=sum0(1,:)+COORD(kPOIN,:);
    end
    IPCOD(NPOIN+iELEM,:)=sum0(1,:)/NNODE;
end

NTVAR=NINTP*NDOFN;
FMATX=zeros(NTVAR,NTVAR);
bvect=zeros(NTVAR,1);
for iINTP=1:NINTP
    xi=IPCOD(iINTP,1);
    yi=IPCOD(iINTP,2);
    % User function is used to provide body forces and
    % temperature
    [GBFOR,TEMPR]=USERFUN(xi,yi,PROPS);
    bvect(2*iINTP-1)=GBFOR(1);
    bvect(2*iINTP)  =GBFOR(2);
    for jINTP=1:NINTP
        xj=IPCOD(jINTP,1);
        yj=IPCOD(jINTP,2);
        r=sqrt((xi-xj)^2+(yi-yj)^2);
        % TPS: r^(2*n)ln(r)
        n=2;
        if 1+r==1
            r=0;
            lnr=0;
        else
            lnr=log(r);
        end
        temp=r^(2*n)*lnr;
        FMATX(2*iINTP-1,2*jINTP-1)=r^(2*n)*lnr;
        FMATX(2*iINTP,  2*jINTP)  =r^(2*n)*lnr;
    end
end
% Solve linear algebraic equations
[CRBFI]=LSSOLVR(FMATX,bvect,NTVAR);