function [CECOD,UCENP,SCENP]=FIEDCEN(NELEM,MATNO,...
    TSELE,CHELE,LNODS,COORD,PROPS,ELNOD,ASDIS)
% Compute internal fields at central point of each
% element
% Input parameters:
%   NELEM: Number of elements in domain
%   MATNO: Material index of each element
%   LNODS: Element connectivity
%   COORD: Coordinates of nodes
%   PROPS: Properties of materials
%   ELNOD: Local relation of edge and nodes
%   ASDIS: Nodal generalized displacement field in DOF
%          order
% Output parameters:
%   CECOD: Coordinates of centroid of each element
%   UCENP: Displacement fields at centroid
%   SCENP: Stress fields at centroid
% ****************************************************
global NDIME NDOFN NNODE NEDGE NODEG NSTRE NMATS NGAUS;

CECOD=zeros(NELEM,NDIME);
UCENP=zeros(NELEM,NDOFN);
SCENP=zeros(NELEM,NSTRE);
% Element loop for internal fields at central point
for iELEM=1:NELEM
    kMATS=MATNO(iELEM);
    kTSEL=TSELE(iELEM);
    CLENG=CHELE(iELEM);
    % Compute some quantities related to each element
    [ECOOD,CenCoord]=ELEPARS(iELEM,LNODS,COORD);    
    % Identify nodal field of the specified element
    [d_Ele]=EDISNOD(iELEM,LNODS,ASDIS);
    % Compute H matrix
    [EHMTX]=HMATRIX(ECOOD,ELNOD,kMATS,kTSEL,...
        CLENG,PROPS);
    % Compute G matrix
    [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,kTSEL,...
        CLENG,PROPS);
    % Calculate the ce coefficients: m by 1
    [c_Ele]=CMATRIX(EHMTX,EGMTX,d_Ele);
    % Recover rigid-body motion vector: 3 by 1
    [c0]=RIGIDRV(ECOOD,c_Ele,d_Ele,kMATS,kTSEL,...
        CLENG,PROPS);
    % Compute internal fields at element central point
    % Note: coordinates of centroid is (0,0) in local
    % element coordinates system
    if kTSEL==0
        xp=0;
        yp=0;
        [N_SET,T_SET]=TREFFTZ(xp,yp,kMATS,PROPS);
    elseif kTSEL==1
        xp=0;
        yp=CLENG;
        [N_SET,T_SET]=SCHTREF(xp,yp,CLENG,kMATS,PROPS);
    end
    GDISP=N_SET*c_Ele;
    GSTRE=T_SET*c_Ele;
    UCENP(iELEM,1)=GDISP(1)+c0(1)+yp*c0(3);
    UCENP(iELEM,2)=GDISP(2)+c0(2)-xp*c0(3);
    SCENP(iELEM,1)=GSTRE(1);
    SCENP(iELEM,2)=GSTRE(2);
    SCENP(iELEM,3)=GSTRE(3);
    % Coordinates of computing points
    CECOD(iELEM,:)=[CenCoord(1)+xp,CenCoord(2)+yp];
end
clear EHMTX EGMTX c_Ele d_Ele N_SET T_SET;