function [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,...
    kTSEL,CLENG,PROPS)
% Compute element matrix G
%   G = integral( QT* Shape) along element boundary
% Input parameters:
%   ECOOD: Coordinates of nodes of specified element
%   ELNOD: Local relation of edge and nodes
%   kMATS: Material number of specified element
%   kTSEL: Type of element
%   CLENG: Characteristic length of special element
%   PROPS: Properties of materials
% Output parameters:
%   EGMTX: Element G matrix
% ****************************************************
global NTREF NDIME NDOFN NEDGE NNODE NGAUS;

% Gaussian point and weight coefficients
[POSGP,WEIGP]=GAUSSQU(NGAUS);
% Material properties
THICK=PROPS(kMATS,3);
% Compute G matrix of every element
NEVAB=NNODE*NDOFN;
EGMTX=zeros(NTREF,NEVAB);
for iEDGE=1:NEDGE
    for iGAUS=1:NGAUS
        EXISP=POSGP(iGAUS);
        [CORGS,DVOLU,AMTRX,SHMTX]=QUANGAS(iEDGE,...
            EXISP,ECOOD,ELNOD);
        DVOLU=DVOLU*WEIGP(iGAUS);
        if THICK+1~=1
            DVOLU=DVOLU*THICK;
        end
        xp=CORGS(1);
        yp=CORGS(2);
        if kTSEL==0
            [N_SET,T_SET]=TREFFTZ(xp,yp,kMATS,PROPS);
        elseif kTSEL==1
            [N_SET,T_SET]=SCHTREF(xp,yp,CLENG,...
                kMATS,PROPS);
        end
        Q_SET=AMTRX*T_SET;
        QTS=Q_SET'*SHMTX;
        for im=1:NTREF
            for jn=1:NEVAB
                EGMTX(im,jn)=EGMTX(im,jn)+...
                    QTS(im,jn)*DVOLU;
            end
        end
    end
end
clear Q_SET QTS N_SET T_SET POSGP WEIGP SHMTX;