function MAINFUN
%** Main function calling other subroutines for solving
%   plane elasticity with general and special elements
%********************************************
global NTREF NTYPE NDIME NDOFN NNODE NEDGE NODEG NSTRE;
global NMATS NPROP NGAUS;

disp('**********************************************');
disp('            Hybrid Trefftz FEM');
disp('        for plane elastic problems');
disp('     with general and special elements');
disp('**********************************************');

% Input data from file
[NPOIN,NELEM,COORD,MATNO,TSELE,CHELE,LNODS,NVFIX,...
    NOFIX,IFPRE,PRESC,NPLOD,NDLEG,LODPT,POINT,NEASS,...
    NOPRS,PRESS,PROPS]=INPUTDT;
% Generate local relations of nodes and edges
[ELNOD]=TYPELEM;
% Element loop for stiffness matrix
NEQNS=NPOIN*NDOFN;
GSTIF=zeros(NEQNS,NEQNS);
GLOAD=zeros(NEQNS,1);
for iELEM=1:NELEM
    kMATS=MATNO(iELEM);
    kTSEL=TSELE(iELEM);
    CLENG=CHELE(iELEM);
    % Compute some quantities of each element
    [ECOOD,CenCoord]=ELEPARS(iELEM,LNODS,COORD);
    % Compute H matrix
    [EHMTX]=HMATRIX(ECOOD,ELNOD,kMATS,kTSEL,...
        CLENG,PROPS);
    % Compute G matrix
    [EGMTX]=GMATRIX(ECOOD,ELNOD,kMATS,kTSEL,...
        CLENG,PROPS);
    % Compute element stiffnesss matrix
    [ESTIF]=KMATRIX(EHMTX,EGMTX);
    % Assemble stiffness matrix
    [GSTIF]=ASMSTIF(iELEM,NNODE,LNODS,ESTIF,GSTIF);
end
% Compute equivalent loads
[GLOAD]=PVECTOR(LNODS,COORD,NDLEG,NEASS,NOPRS,...
    PRESS,NPLOD,LODPT,POINT,GLOAD);
% Introduce constrained displacements and point loads
[GSTIF,GLOAD]=INDISBC(NEQNS,NVFIX,NOFIX,IFPRE,PRESC,...
    GSTIF,GLOAD);
% Solve linear system of equations and store
% displacements of each node in the array ASDIS
[ASDIS]=LSSOLVR(GSTIF,GLOAD,NEQNS);
% Output nodal potential
[UPOIN]=FIEDNOD(NPOIN,ASDIS);
% Compute potential and flux components at central
% point of element
[CECOD,UCENP,SCENP]=FIEDCEN(NELEM,MATNO,TSELE,CHELE,...
    LNODS,COORD,PROPS,ELNOD,ASDIS);
% Output results
OPRESUT(NPOIN,COORD,UPOIN,NELEM,CECOD,UCENP,SCENP,...
    NVFIX,NPLOD,NDLEG);
disp('--- All procedures are finished ---');