function OPRESUT(NPOIN,COORD,UPOIN,NELEM,CECOD,...
    UCENP,SCENP,NVFIX,NPLOD,NDLEG)
% Output of results to a file
% Input parameters:
%   NPOIN: Number of nodes in domain
%   COORD: Coordinates of nodes
%   UPOIN: Nodal displacement field
%   NELEM: Number of elements in domain
%   CECOD: Coordinates of centroid of each element
%   UCENP: Displacement fields at centroid
%   SCENP: Stress fields at centroid
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted
%   NPLOD: Number of concentrated loads
%   NDLEG: Number of loaded edges
% Output parameters: No
% **********************************
global NTREF NTYPE NDIME NDOFN NNODE NEDGE NODEG NSTRE;
global NMATS NPROP NGAUS;

fp=fopen('Result.txt','wt');

fprintf(fp,'**Basic parameters\n');
fprintf(fp,'\n');
fprintf(fp,'NTREF=%5d  NTYPE=%5d\n',NTREF,NTYPE);
fprintf(fp,'\n');
fprintf(fp,'NDIME=%5d  NDOFN=%5d  NSTRE=%5d\n',...
    NDIME,NDOFN,NSTRE);
fprintf(fp,'\n');
fprintf(fp,'NNODE=%5d  NEDGE=%5d  NODEG=%5d\n',...
    NNODE,NEDGE,NODEG);
fprintf(fp,'\n');
fprintf(fp,'NMATS=%5d  NPROP=%5d  NGAUS=%5d\n',...
    NMATS,NPROP,NGAUS);
fprintf(fp,'\n');
fprintf(fp,'NPOIN=%5d  NELEM=%5d  NVFIX=%5d\n',...
    NPOIN,NELEM,NVFIX);
fprintf(fp,'\n');
fprintf(fp,'NPLOD=%5d  NDLEG=%5d\n',NPLOD,NDLEG);
fprintf(fp,'\n');
fprintf(fp,'\n');
fprintf(fp,'**Generalized displacements at nodes\n');
fprintf(fp,'-------------------------------------\n');
fprintf(fp,'Node# COD#1-->#NDIME DISPL#1-->#NODFN\n');
fprintf(fp,'-------------------------------------\n');
for iPOIN=1:NPOIN
    xp=COORD(iPOIN,1);
    yp=COORD(iPOIN,2);
    fprintf(fp,'%5d  %9.4f  %9.4f',iPOIN,xp,yp);
    for iDOFN=1:NDOFN
        fprintf(fp,'  %9.4f',UPOIN(iPOIN,iDOFN));
    end
    fprintf(fp,'\n');   
end
fprintf(fp,'\n');
fprintf(fp,'\n');
fprintf(fp,...
   '**Generalized Displacement at Element Centroid\n');
fprintf(fp,'-------------------------------------\n');
fprintf(fp,'Elem# COD#1-->#NDIME DISPL#1-->#NODFN\n');
fprintf(fp,'-------------------------------------\n');
for iELEM=1:NELEM
    xp=CECOD(iELEM,1);
    yp=CECOD(iELEM,2);
    fprintf(fp,'%5d  %9.4f  %9.4f',iELEM,xp,yp);
    for iDOFN=1:NDOFN
        fprintf(fp,'  %9.4f',UCENP(iELEM,iDOFN));
    end
    fprintf(fp,'\n');
end
fprintf(fp,'\n');
fprintf(fp,...
    '**Generalized Stress at Element Centroid\n');
fprintf(fp,'-------------------------\n');
fprintf(fp,'Elem#    STRES#1-->#NSTRE\n');
fprintf(fp,'-------------------------\n');
for iELEM=1:NELEM
    fprintf(fp,'%5d',iELEM);
    for iSTRE=1:NSTRE
        fprintf(fp,'  %9.4f',SCENP(iELEM,iSTRE));
    end
    fprintf(fp,'\n');
end
fclose(fp);