function [N_SET,T_SET]=SCHTREF(sp,tp,b,kMATS,PROPS)
% Compute Trefftz functions of special circular hole
% element
% Input parameters:
%   sp,tp: Coordinates
%   CLENG: Characteristic length
%   kMATS: Material number
%   PROPS: Material properties
% Output parameters:
%   N_SET: Displacements Trefftz functions
%   T_SET: Stresses Trefftz functions
% ****************************************************
global NTREF NTYPE NNODE NDOFN;

% Check the number of terms of Trefftz functions
temp=NNODE*NDOFN-3;
if NTREF<temp
    error('Too small terms of Trefftz functions!');
end
% Initializing N and T
N_SET=zeros(2,NTREF);
T_SET=zeros(3,NTREF);
% Material properties
YOUNG=PROPS(kMATS,1);
POISS=PROPS(kMATS,2);
if NTYPE==1 % plane stress
    E=YOUNG;
    nu=POISS;
elseif NTYPE==2 % plane strain
    E=YOUNG/(1-POISS^2);
    nu=POISS/(1-POISS);
end
% Evaluate r ans s
r=sqrt(sp^2+tp^2);
s=atan2(tp,sp);
% k=0
k=0;
A0=(1+nu)/(1-nu)*b^2;

DF0=A0/r+r;
DG0=0;
SF0=E/(1-nu)-E/(1+nu)*A0/r^2;
SG0=E/(1-nu)+E/(1+nu)*A0/r^2;
SH0=0;

N_SET(1,1)=DF0;
N_SET(2,1)=DG0;
T_SET(1,1)=SF0;
T_SET(2,1)=SG0;
T_SET(3,1)=SH0;
% k=1
k=1;
A1=(1+nu)/(1-3*nu)*b^4;
A2=(5+nu)/(1-3*nu);
D1=-2*E*A1/(1+nu);
D2=E*(2+nu+nu*A2)/(1-nu^2);
D3=E*(1+2*nu+A2)/(1-nu^2);
D4=E*(-1+A2)/2/(1+nu);

DF1=A1/r^2+r^2;
DG1=A1/r^2+A2*r^2;
SF1= D1*r^(-3)+D2*r;
SG1=-D1*r^(-3)+D3*r;
SH1= D1*r^(-3)+D4*r;

N_SET(1,2)=DF1*cos(s);
N_SET(2,2)=DG1*sin(s);
T_SET(1,2)=SF1*cos(s);
T_SET(2,2)=SG1*cos(s);
T_SET(3,2)=SH1*sin(s);

N_SET(1,3)= DF1*sin(s);
N_SET(2,3)=-DG1*cos(s);
T_SET(1,3)= SF1*sin(s);
T_SET(2,3)= SG1*sin(s);
T_SET(3,3)=-SH1*cos(s);

NK(1)=0;
NK(2:3)=1;

nt=4;
% k>=2
while nt<NTREF
    k=k+1;
    
    B1=(2*(1-nu)+k*(1+nu))/((k-1)*(2*(1-nu)-k*(1+nu)))*b^(2*k);
    B2=k^2*(1+nu)/((k-1)*(2*(1-nu)-k*(1+nu)))*b^2;
    B3=(2*(1-nu)+k*(1+nu))/(1-k)/(1+nu)*b^(-2);
    B4=1/(1-k)*b^(-2*k);
    B5=(k*(1+nu)-4)/((k-1)*(2*(1-nu)-k*(1+nu)))*b^(2*k);
    B6=(k*(1+nu)+4)/(2*(1-nu)-k*(1+nu));
    B7=(k*(1+nu)-4)/(1-k)/(1+nu)*b^(-2);
    J1=E*((1+nu-k)*B1+nu*k*B5)/(1-nu^2);
    J2=E*(k-1)*B2/(1+nu);
    J3=E*((1+nu+k)+nu*k*B6)/(1-nu^2);
    J4=E*((1+nu-nu*k)*B1+k*B5)/(1-nu^2);
    J5=E*((1+nu+nu*k)+k*B6)/(1-nu^2);
    J6=-E*k*(B1+B5)/2/(1+nu);
    J7=E*k*(B6-1)/2/(1+nu);
    J8=E*((1+nu-k)*B3+nu*k*B7)/(1-nu^2);
    J9=E*(k-1)*B4/(1+nu);
    J10=-E*(1+k)/(1+nu);
    J11=E*((1+nu-nu*k)*B3+k*B7)/(1-nu^2);
    J12=-E*k*(B3+B7)/2/(1+nu);
    
    DF2=B1*r^(1-k)+B2*r^(k-1)+r^(1+k);
    DG2=B5*r^(1-k)-B2*r^(k-1)+B6*r^(1+k);
    DF3=B3*r^(1-k)+B4*r^(k-1)+r^(-1-k);
    DG3=B7*r^(1-k)-B4*r^(k-1)+r^(-1-k);
    SF2=J1*r^(-k)+J2*r^(k-2)+J3*r^(k);
    SG2=J4*r^(-k)-J2*r^(k-2)+J5*r^(k);
    SH2=J6*r^(-k)-J2*r^(k-2)+J7*r^(k);
    SF3=J8* r^(-k)+J9*r^(k-2)+J10*r^(-2-k);
    SG3=J11*r^(-k)-J9*r^(k-2)-J10*r^(-2-k);
    SH3=J12*r^(-k)-J9*r^(k-2)+J10*r^(-2-k);
    
    N_SET(1,nt)=DF2*cos(k*s);
    N_SET(2,nt)=DG2*sin(k*s);
    T_SET(1,nt)=SF2*cos(k*s);
    T_SET(2,nt)=SG2*cos(k*s);
    T_SET(3,nt)=SH2*sin(k*s);
    NK(nt)=k;
    
    nt=nt+1;
    N_SET(1,nt)= DF2*sin(k*s);
    N_SET(2,nt)=-DG2*cos(k*s);
    T_SET(1,nt)= SF2*sin(k*s);
    T_SET(2,nt)= SG2*sin(k*s);
    T_SET(3,nt)=-SH2*cos(k*s);
    NK(nt)=k;
    
    nt=nt+1;
    if nt>=NTREF
        break;
    end
    N_SET(1,nt)=DF3*cos(k*s);
    N_SET(2,nt)=DG3*sin(k*s);
    T_SET(1,nt)=SF3*cos(k*s);
    T_SET(2,nt)=SG3*cos(k*s);
    T_SET(3,nt)=SH3*sin(k*s);
    NK(nt)=k;
    
    nt=nt+1;
    N_SET(1,nt)= DF3*sin(k*s);
    N_SET(2,nt)=-DG3*cos(k*s);
    T_SET(1,nt)= SF3*sin(k*s);
    T_SET(2,nt)= SG3*sin(k*s);
    T_SET(3,nt)=-SH3*cos(k*s);
    NK(nt)=k;
    
    nt=nt+1;
end
% Transformation of coordinates
DM=[cos(s),-sin(s);sin(s),cos(s)];
SM=[(cos(s))^2,(sin(s))^2,-sin(2*s);...
    (sin(s))^2,(cos(s))^2,sin(2*s);...
    0.5*sin(2*s),-0.5*sin(2*s),cos(2*s)];
N_SET=DM*N_SET;
T_SET=SM*T_SET;