function [GK,GP]=INDISBC(NEQNS,NVFIX,NOFIX,IFPRE,...
    PRESC,GK,GP)
% ** Modify global stiffness matrix GK and
%    equivalent load vector GP by the penalty
%    approach
% Input parameters:
%   NEQNS: Number of equations
%   NVFIX: Number of boundary nodes at which specified
%          DOF is restricted
%   NOFIX: Global index of nodes at which specified DOF
%          is restricted
%   IFPRE: Types of constraints of each DOF
%   PRESC: Specified values
%   GK   : Global stiffness matrix
%   GP   : Global equivalent nodal load vector
% Output parameters:
%   GK   : Modified global stiffness matrix
%   GP   : Modified global equivalent nodal load vector
% **********************************

global NDOFN;

if NVFIX>0
    % Decide penalty parameter CNST
    CNST=max(max(abs(GK)));
    if CNST+1==1
        error('**Singular stiffness matrix GK!');
    end
    CNST=CNST*1000000;
    % Modify GK and GP for specified nodal
    % displacements
    for iVFIX=1:NVFIX
        kPOIN=NOFIX(iVFIX);
        for iDOFN=1:NDOFN
            NR=(kPOIN-1)*NDOFN+iDOFN;
            ii=IFPRE(iVFIX,iDOFN);
            % unit value indicates a constrained DOF
            if ii==1
                disv=PRESC(iVFIX,iDOFN);
                GK(NR,NR)=GK(NR,NR)+CNST;
                GP(NR)=GP(NR)+CNST*disv;
            end
        end
    end
end