function [N_SET,T_SET]=TREFFTZ(sp,tp,kMATS,PROPS)
% Compute the Trefftz functions with specified terms
% Input parameters:
%   sp,tp: coordinates
%   kMATS:  Material number
%   PROPS:  Material properties
% Output parameters:
%   N_SET: Displacements Trefftz functions
%   T_SET: Stresses Trefftz functions
% ************************************************
global NTREF NTYPE NNODE NDOFN;

% Check the number of terms of Trefftz functions
temp=NNODE*NDOFN-3;
if NTREF<temp
    error('Too small terms of Trefftz functions!');
end

YOUNG=PROPS(kMATS,1);
POISS=PROPS(kMATS,2);
if NTYPE==1 % plane stress
    KAMA=(3.0-POISS)/(1.0+POISS);
elseif NTYPE==2 % plane strain
    KAMA=3.0-4*POISS;
end
GMOD=YOUNG/(2*(1+POISS));

Z=complex(sp,tp);
ZC=conj(Z);
UNIT=complex(0,1);
% Trefftz functions
ZK=zeros(1,NTREF);
RK=zeros(1,NTREF);
SK=zeros(1,NTREF);
NK=zeros(1,NTREF);

k=1;
ZK(1)=KAMA*Z-Z; % KAMA*Z-Z
ZK(2)=UNIT*ZC;  % I*ZC
ZK(3)=-ZC;      % -ZC

RK(1)=2;        % 2
RK(2)=0;
RK(3)=0;

SK(1)=0;        % 0
SK(2)=UNIT;     % I
SK(3)=1;        % 1

NK(1:3)=k;
nt=4;
while nt<NTREF
    k=k+1;
    
    ZK(nt)=KAMA*UNIT*Z^k+k*UNIT*Z*ZC^(k-1);
    RK(nt)=2*UNIT*k*Z^(k-1);
    SK(nt)=UNIT*k*(k-1)*Z^(k-2)*ZC;
    NK(nt)=k;
    
    nt=nt+1;
    ZK(nt)=KAMA*Z^k-k*Z*ZC^(k-1);
    RK(nt)=2*k*Z^(k-1);
    SK(nt)=k*(k-1)*Z^(k-2)*ZC;
    NK(nt)=k;
    
    nt=nt+1;
    if nt>=NTREF
        break;
    end
    ZK(nt)=UNIT*ZC^k;
    RK(nt)=0;
    SK(nt)=UNIT*k*Z^(k-1);
    NK(nt)=k;
    
    nt=nt+1;
    ZK(nt)=-ZC^k;
    RK(nt)=0;
    SK(nt)=k*Z^(k-1);
    NK(nt)=k;
    
    nt=nt+1;
end

% Compute Trefftz functions N_SET and T_SET
N_SET=zeros(2,NTREF);
T_SET=zeros(3,NTREF);
for im=1:NTREF
    N_SET(1,im)=real(ZK(im))/(2*GMOD);
    N_SET(2,im)=imag(ZK(im))/(2*GMOD);
    k=NK(im);
    n1=4*(k-1)-3;
    n2=4*(k-1)-2;
    km=im-3;
    if (im==1)|(km==n1)|(km==n2)
        T_SET(1,im)=real(RK(im))-real(SK(im));
        T_SET(2,im)=real(RK(im))+real(SK(im));
        T_SET(3,im)=imag(SK(im));
    else
        T_SET(1,im)=-real(SK(im));
        T_SET(2,im)= real(SK(im));
        T_SET(3,im)= imag(SK(im));
    end
end